#!/usr/bin/env python

"""
Test CIM enumerate performance with a defined class that can be controlled
from the provider TST_ResponseStressTestCxx. This is limited to OpenPegasus
because only OpenPegasus implements the defined class.
"""

from __future__ import absolute_import
import sys as _sys
# pylint: disable=missing-docstring,superfluous-parens,no-self-use
import datetime
# pylint: disable=unused-import
import warnings  # noqa F401

import argparse as _argparse
from argparse import RawTextHelpFormatter

import getpass as _getpass
import re

from pywbem import WBEMConnection, Error, Uint64

# Pegasus class/namespace to use for test
TEST_NAMESPACE = "test/TestProvider"
TEST_CLASSNAME = "TST_ResponseStressTestCxx"

TABLE_FORMAT = '%-10.10s %-10.10s %-10.10s %-10.10s %-10.10s %-11.11s %-10.10s'


def set_provider_parameters(conn, count, size):
    """Set the provider parameters with InvokeMethod"""
    try:
        result = conn.InvokeMethod("Set", TEST_CLASSNAME,
                                   [('ResponseCount', Uint64(count)),
                                    ('Size', Uint64(size))])

        if result[0] != 0:
            print('SendTestIndicationCount Method error. Nonzero return=%s'
                  % result[0])
            return False
        return True

    except Error as er:
        print('Error: Invoke Method exception %s' % er)
        raise


def run_enum_instances(conn):
    """Execute an EnumerateInstances request and return the instances"""
    instances = conn.EnumerateInstances(TEST_CLASSNAME)
    return len(instances)


def run_pull_enum_instances(conn, max_object_count):
    """Execute an open request followed by pull requests"""
    op_count = 0
    result = conn.OpenEnumerateInstances(
        TEST_CLASSNAME,
        MaxObjectCount=max_object_count)
    insts_pulled = result.instances

    op_count += 1

    while not result.eos:
        result = conn.PullInstancesWithPath(result.context,
                                            MaxObjectCount=max_object_count)
        op_count += 1
        insts_pulled.extend(result.instances)

    # return tuple containing info on the sequence
    return [len(insts_pulled), op_count]


def run_single_test(conn, response_count, response_size, max_obj_cnt_array):
    """run a single test for a defined response_count and response size"""

    set_provider_parameters(conn, response_count, response_size)

    start_time = datetime.datetime.now()
    enum_count = run_enum_instances(conn)
    enum_time = datetime.datetime.now() - start_time
    inst_per_sec = '%06.2f' % (enum_count / enum_time.total_seconds())
    print(TABLE_FORMAT % ('Enum', enum_count, response_size,
                          'na', 'na', enum_time, inst_per_sec))

    for max_obj_cnt in max_obj_cnt_array:
        pull_start_time = datetime.datetime.now()
        pull_result = run_pull_enum_instances(conn, max_obj_cnt)
        pull_time = datetime.datetime.now() - pull_start_time
        inst_per_sec = '%06.2f' % (pull_result[0] / pull_time.total_seconds())

        print(TABLE_FORMAT % ('Open/Pull', pull_result[0], response_size,
                              max_obj_cnt, pull_result[1], pull_time,
                              inst_per_sec))


def run_tests(conn):
    """Run test based on limits provided"""

    print(TABLE_FORMAT % ('Operation', 'Response', 'RespSize', 'MaxObjCnt',
                          'Request', 'Time', 'inst/sec'))
    print(TABLE_FORMAT % ('', 'Count', 'Bytes', 'Request',
                          'Count', '', ''))
    for response_size in [100, 1000, 10000]:
        for response_count in [1, 100, 1000, 10000, 100000]:
            pull_sizes = [1, 100, 1000]
            run_single_test(conn, response_count, response_size, pull_sizes)


def main(prog):
    """
    Parse command line arguments, connect to the WBEM server and open the
    interactive shell.
    """

    usage = '%(prog)s [options] server'
    desc = """
Provide performance information on EnumerateInstances vs. Open & Pull
enumeration sequences responses.

Generates a table of response times for an array of max_object_count,
response sizes, etc.values.
"""
    epilog = """
Examples:
  %s https://localhost:15345 -n vendor -u sheldon -p penny
          - (https localhost, port=15345, namespace=vendor user=sheldon
         password=penny)

  %s http://[2001:db8::1234-eth0] -(http port 5988 ipv6, zone id eth0)
""" % (prog, prog)

    argparser = _argparse.ArgumentParser(
        prog=prog, usage=usage, description=desc, epilog=epilog,
        add_help=False, formatter_class=RawTextHelpFormatter)

    pos_arggroup = argparser.add_argument_group(
        'Positional arguments')
    pos_arggroup.add_argument(
        'server', metavar='server', nargs='?',
        help='Host name or url of the WBEM server in this format:\n'
             '    [{scheme}://]{host}[:{port}]\n'
             '- scheme: Defines the protocol to use;\n'
             '    - "https" for HTTPs protocol\n'
             '    - "http" for HTTP protocol.\n'
             '  Default: "https".\n'
             '- host: Defines host name as follows:\n'
             '     - short or fully qualified DNS hostname,\n'
             '     - literal IPV4 address(dotted)\n'
             '     - literal IPV6 address (RFC 3986) with zone\n'
             '       identifier extensions(RFC 6874)\n'
             '       supporting "-" or %%25 for the delimiter.\n'
             '- port: Defines the WBEM server port to be used\n'
             '  Defaults:\n'
             '     - HTTP  - 5988\n'
             '     - HTTPS - 5989\n')

    server_arggroup = argparser.add_argument_group(
        'Server related options',
        'Specify the WBEM server namespace and timeout')
    server_arggroup.add_argument(
        '-t', '--timeout', dest='timeout', metavar='timeout', type=int,
        default=None,
        help='Timeout of the completion of WBEM Server operation\n'
             'in seconds(integer between 0 and 300).\n'
             'Default: No timeout')

    security_arggroup = argparser.add_argument_group(
        'Connection security related options',
        'Specify user name and password or certificates and keys')
    security_arggroup.add_argument(
        '-u', '--user', dest='user', metavar='user',
        help='User name for authenticating with the WBEM server.\n'
             'Default: No user name.')
    security_arggroup.add_argument(
        '-p', '--password', dest='password', metavar='password',
        help='Password for authenticating with the WBEM server.\n'
             'Default: Will be prompted for, if user name\nspecified.')
    security_arggroup.add_argument(
        '-nvc', '--no-verify-cert', dest='no_verify_cert',
        action='store_true',
        help='Client will not verify certificate returned by the WBEM\n'
             'server (see cacerts). This bypasses the client-side\n'
             'verification of the server identity, but allows\n'
             'encrypted communication with a server for which the\n'
             'client does not have certificates.')
    security_arggroup.add_argument(
        '--cacerts', dest='ca_certs', metavar='cacerts',
        help='File or directory containing certificates that will be\n'
             'matched against a certificate received from the WBEM\n'
             'server. Set the --no-verify-cert option to bypass\n'
             'client verification of the WBEM server certificate.\n'
             'Default: Searches for matching certificates in the\n'
             'following system directories:\n')

    security_arggroup.add_argument(
        '--certfile', dest='cert_file', metavar='certfile',
        help='Client certificate file for authenticating with the\n'
             'WBEM server. If option specified the client attempts\n'
             'to execute mutual authentication.\n'
             'Default: Simple authentication.')
    security_arggroup.add_argument(
        '--keyfile', dest='key_file', metavar='keyfile',
        help='Client private key file for authenticating with the\n'
             'WBEM server. Not required if private key is part of the\n'
             'certfile option. Not allowed if no certfile option.\n'
             'Default: No client key file. Client private key should\n'
             'then be part  of the certfile')

    general_arggroup = argparser.add_argument_group(
        'General options')
    general_arggroup.add_argument(
        '-v', '--verbose', dest='verbose',
        action='store_true', default=False,
        help='Print more messages while processing')
    general_arggroup.add_argument(
        '-h', '--help', action='help',
        help='Show this help message and exit')

    opts = argparser.parse_args()

    if not opts.server:
        argparser.error('No WBEM server specified')

    if opts.server[0] == '/':
        url = opts.server

    elif re.match(r"^https{0,1}://", opts.server) is not None:
        url = opts.server

    elif re.match(r"^[a-zA-Z0-9]+://", opts.server) is not None:
        argparser.error('Invalid scheme on server argument.'
                        ' Use "http" or "https"')

    else:
        url = '%s://%s' % ('https', opts.server)

    creds = None

    if opts.key_file is not None and opts.cert_file is None:
        argparser.error('keyfile option requires certfile option')

    if opts.user is not None and opts.password is None:
        opts.password = _getpass.getpass('Enter password for %s: '
                                         % opts.user)

    if opts.user is not None or opts.password is not None:
        creds = (opts.user, opts.password)

    # if client cert and key provided, create dictionary for
    # wbem connection
    x509_dict = None
    if opts.cert_file is not None:
        x509_dict = {"cert_file": opts.cert_file}
        if opts.key_file is not None:
            x509_dict.update({'key_file': opts.key_file})

    conn = WBEMConnection(url, creds, default_namespace=TEST_NAMESPACE,
                          no_verification=True,
                          x509=x509_dict, ca_certs=opts.ca_certs,
                          timeout=opts.timeout)

    run_tests(conn)

    return 0


if __name__ == '__main__':
    _sys.exit(main('run_enum_performance.py'))



#!/usr/bin/env python

"""
Unit test recorder functions.

"""

from __future__ import absolute_import, print_function

# Allows use of lots of single character variable names.
# pylint: disable=invalid-name,missing-docstring,too-many-statements
# pylint: disable=too-many-lines,no-self-use
from datetime import timedelta, datetime, tzinfo
import unittest
import os
import os.path
import logging
from io import open as _open
import yaml
import six
from testfixtures import LogCapture, log_capture

from pywbem import CIMInstanceName, CIMInstance, MinutesFromUTC, \
    Uint8, Uint16, Uint32, Uint64, Sint8, Sint16, \
    Sint32, Sint64, Real32, Real64, CIMProperty, CIMDateTime, CIMError, \
    HTTPError
# Renamed the following import to not have py.test pick it up as a test class:
from pywbem import TestClientRecorder as _TestClientRecorder
from pywbem import LogOperationRecorder as _LogOperationRecorder
from pywbem import PywbemLoggers, WBEMConnection

# used to build result tuple for test
from pywbem.cim_operations import pull_path_result_tuple, pull_inst_result_tuple

# test outpuf file for the recorder tests.  This is opened for each
# test to save yaml output and may be reloaded during the same test
# to confirm the yaml results.
TEST_YAML_FILE = 'test_recorder.yaml'
SCRIPT_DIR = os.path.dirname(__file__)

LOG_FILE_NAME = 'test_recorder.log'
TEST_OUTPUT_LOG = '%s/%s' % (SCRIPT_DIR, LOG_FILE_NAME)

VERBOSE = False


class BaseRecorderTests(unittest.TestCase):
    """Base class for recorder unit tests. Implements method
       for creating instance
    """

    def create_ciminstance(self):
        """
        Create a sample instance with multiple properties and
        property types.
        """
        class TZ(tzinfo):
            """'Simplistic tsinfo subclass for this test"""
            def utcoffset(self, dt):  # pylint: disable=unused-argument
                return timedelta(minutes=-399)

        dt = datetime(year=2016, month=3, day=31, hour=19, minute=30,
                      second=40, microsecond=654321,
                      tzinfo=MinutesFromUTC(120))
        cim_dt = CIMDateTime(dt)
        props_input = {
            'S1': b'Ham',
            'Bool': True,
            'UI8': Uint8(42),
            'UI16': Uint16(4216),
            'UI32': Uint32(4232),
            'UI64': Uint64(4264),
            'SI8': Sint8(-42),
            'SI16': Sint16(-4216),
            'SI32': Sint32(-4232),
            'SI64': Sint64(-4264),
            'R32': Real32(42.0),
            'R64': Real64(42.64),
            'DTI': CIMDateTime(timedelta(10, 49, 20)),
            'DTF': cim_dt,
            'DTP': CIMDateTime(datetime(2014, 9, 22, 10, 49, 20, 524789,
                                        tzinfo=TZ())),
        }
        # TODO python 2.7 will not write the following unicode character
        # For the moment, only add this property in python 3 test
        if six.PY3:
            props_input['S2'] = u'H\u00E4m'  # U+00E4 = lower case a umlaut

        inst = CIMInstance('CIM_Foo', props_input)
        return inst

    def create_ciminstancename(self):
        kb = {'Chicken': 'Ham'}
        obj_name = CIMInstanceName('CIM_Foo',
                                   kb,
                                   namespace='root/cimv2',
                                   host='woot.com')
        return obj_name


class ClientRecorderTests(BaseRecorderTests):
    """
    Common base for all tests on the TestClientRecorder. Defines specific common
    methods including setUp and tearDown for the TestClientRecorder.
    """
    def setUp(self):
        """ Setup recorder instance including defining output file"""
        self.testyamlfile = os.path.join(SCRIPT_DIR, TEST_YAML_FILE)
        if os.path.isfile(self.testyamlfile):
            os.remove(self.testyamlfile)

        self.yamlfp = _TestClientRecorder.open_file(self.testyamlfile, 'a')

        self.test_recorder = _TestClientRecorder(self.yamlfp)
        self.test_recorder.reset()
        self.test_recorder.enable()

    def tearDown(self):
        """Close the test_client YAML file."""
        if self.yamlfp is not None:
            self.yamlfp.close()

    def closeYamlFile(self):
        """Close the yaml file if it is open"""
        if self.yamlfp is not None:
            self.yamlfp.close()
            self.yamlfp = None

    def loadYamlFile(self):
        """Load any created yaml file"""
        self.closeYamlFile()
        with _open(self.testyamlfile, encoding="utf-8") as fp:
            testyaml = yaml.load(fp)
        return testyaml


class ToYaml(ClientRecorderTests):
    """Test the toyaml function with multiple data input"""
    def test_inst_to_yaml_simple(self):
        """Test Simple instancename toyaml conversion"""

        test_yaml = self.test_recorder.toyaml(self.create_ciminstancename())

        self.assertEqual(test_yaml['pywbem_object'], 'CIMInstanceName')
        self.assertEqual(test_yaml['classname'], 'CIM_Foo')
        self.assertEqual(test_yaml['namespace'], 'root/cimv2')
        kb = test_yaml['keybindings']
        self.assertEqual(kb['Chicken'], 'Ham')

        # CIMClass, cimqualifierdecl

    def test_to_yaml_simple2(self):
        """Test Simple cimdatetime and other primitive types to toyaml"""
        test_yaml = self.test_recorder.toyaml(
            CIMDateTime('20140924193040.654321+120'))
        self.assertEqual(test_yaml, '20140924193040.654321+120')

        self.assertEqual(self.test_recorder.toyaml(True), True)
        self.assertEqual(self.test_recorder.toyaml(False), False)
        self.assertEqual(self.test_recorder.toyaml(1234), 1234)
        self.assertEqual(self.test_recorder.toyaml('blahblah '), 'blahblah ')

    def test_inst_to_yaml_all_prop_types(self):
        """Test all property types toyaml"""

        inst = self.create_ciminstance()

        test_yaml = self.test_recorder.toyaml(inst)

        self.assertEqual(test_yaml['pywbem_object'], 'CIMInstance')
        self.assertEqual(test_yaml['classname'], 'CIM_Foo')
        properties = test_yaml['properties']

        si64 = properties['SI64']
        self.assertEqual(si64['name'], 'SI64')
        self.assertEqual(si64['type'], 'sint64')
        self.assertEqual(si64['value'], -4264)

        dtp = properties['DTP']
        self.assertEqual(dtp['name'], 'DTP')
        self.assertEqual(dtp['type'], 'datetime')
        self.assertEqual(dtp['value'], '20140922104920.524789-399')
        dti = properties['DTI']
        self.assertEqual(dti['name'], 'DTI')
        self.assertEqual(dti['type'], 'datetime')
        self.assertEqual(dti['value'], '00000010000049.000020:000')

        # TODO reference properties
        # TODO embedded object property

    def test_inst_to_yaml_array_props(self):
        """Test  property with array toyaml"""
        str_data = "The pink fox jumped over the big blue dog"
        dt = datetime(2014, 9, 22, 10, 49, 20, 524789)
        array_props = {'MyString': str_data,
                       'MyUint8Array': [Uint8(1), Uint8(2)],
                       'MySint8Array': [Sint8(1), Sint8(2)],
                       'MyUint64Array': [Uint64(123456789),
                                         Uint64(123456789),
                                         Uint64(123456789)],
                       'MyUint32Array': [Uint32(9999), Uint32(9999)],
                       'MyDateTimeArray': [dt, dt, dt],
                       'MyStrLongArray': [str_data, str_data, str_data]}
        inst = CIMInstance('CIM_FooArray', array_props)
        test_yaml = self.test_recorder.toyaml(inst)

        self.assertEqual(test_yaml['pywbem_object'], 'CIMInstance')
        self.assertEqual(test_yaml['classname'], 'CIM_FooArray')
        properties = test_yaml['properties']
        my_string = properties['MyString']
        self.assertEqual(my_string['name'], 'MyString')
        self.assertEqual(my_string['type'], 'string')
        self.assertEqual(my_string['value'], str_data)

        my_uint8array = properties['MyUint8Array']
        self.assertEqual(my_uint8array['name'], 'MyUint8Array')
        self.assertEqual(my_uint8array['type'], 'uint8')
        self.assertEqual(my_uint8array['value'], [Uint8(1), Uint8(2)])

        my_sint8array = properties['MySint8Array']
        self.assertEqual(my_sint8array['name'], 'MySint8Array')
        self.assertEqual(my_sint8array['type'], 'sint8')
        self.assertEqual(my_sint8array['value'], [Sint8(1), Sint8(2)])

        my_sint64array = properties['MyUint64Array']
        self.assertEqual(my_sint64array['name'], 'MyUint64Array')
        self.assertEqual(my_sint64array['type'], 'uint64')
        self.assertEqual(my_sint64array['value'], [Uint64(123456789),
                                                   Uint64(123456789),
                                                   Uint64(123456789)])

        my_datetimearray = properties['MyDateTimeArray']
        self.assertEqual(my_datetimearray['name'], 'MyDateTimeArray')
        self.assertEqual(my_datetimearray['type'], 'datetime')
        cim_dt = str(CIMDateTime(dt))
        self.assertEqual(my_datetimearray['value'], [cim_dt, cim_dt, cim_dt])

    def test_instname_to_yaml(self):
        """Test  instname toyaml"""

        inst_name = self.create_ciminstancename()

        test_yaml = self.test_recorder.toyaml(inst_name)
        self.assertEqual(test_yaml['pywbem_object'], 'CIMInstanceName')
        self.assertEqual(test_yaml['classname'], 'CIM_Foo')
        # TODO host does not appear in output yaml
        # ##self.assertEqual[test_yaml['host'], 'woot.com']
        kbs = test_yaml['keybindings']
        self.assertEqual(len(kbs), 1)
        self.assertEqual(kbs['Chicken'], 'Ham')

    def test_openreq_resulttuple(self):
        """test tuple result from open operation. The input is a
        named tuple.
        """
        result = []
        context = ('test_rtn_context', 'root/cim_namespace')
        result_tuple = pull_path_result_tuple(result, True, None)
        test_yaml = self.test_recorder.toyaml(result_tuple)

        self.assertEqual(test_yaml['paths'], [])
        self.assertEqual(test_yaml['eos'], True)
        self.assertEqual(test_yaml['context'], None)

        result_tuple = pull_path_result_tuple(result, False, context)
        test_yaml = self.test_recorder.toyaml(result_tuple)

        self.assertEqual(test_yaml['paths'], [])
        self.assertEqual(test_yaml['eos'], False)
        self.assertEqual(test_yaml['context'], list(context))


class LogOperationStageTests(ClientRecorderTests):
    """
    Test staging for different cim_operations.  This defines fixed
    parameters for the before and after staging, stages (which creates
    a yaml file), and then inspects that file to determine if valid
    yaml was created
    """

    def test_err_type(self):
        """Test for timedelta stage.  This is one that causes error"""

        obj_name = self.create_ciminstancename()

        # TODO the following fails because of issues with timedelta
        # params = [('Date2', timedelta(60))]
        params = [('StringParam', 'Spotty')]
        # TODO fix the error we have masked above
        self.test_recorder.stage_pywbem_args(method='InvokeMethod',
                                             MethodName='Blah',
                                             ObjectName=obj_name,
                                             Params=params)
        method_result_tuple = None
        method_exception = None
        self.test_recorder.stage_pywbem_result(method_result_tuple,
                                               method_exception)
        # records everything staged to file
        self.test_recorder.record_staged()

        test_yaml = self.loadYamlFile()
        test_yaml = test_yaml[0]
        pywbem_request = test_yaml['pywbem_request']
        self.assertEqual(pywbem_request['url'], 'http://acme.com:80')
        operation = pywbem_request['operation']
        self.assertEqual(operation['pywbem_method'], 'InvokeMethod')
        # TODO test result  This test goes away when we fix timedelta yaml

    def test_invoke_method(self):
        """
        Emulates call to invokemethod to test parameter processing.
        Currently creates the pywbem_request component.
        Each test emulated a single cim operation with fixed data to
        create the input for the yaml, create the yaml, and test the result
        """
        obj_name = self.create_ciminstancename()

        params = [('StringParam', 'Spotty'),
                  ('Uint8', Uint8(1)),
                  ('Sint8', Sint8(2)),
                  ('Uint16', Uint16(3)),
                  ('Sint16', Sint16(3)),
                  ('Uint32', Uint32(4)),
                  ('Sint32', Sint32(5)),
                  ('Uint64', Uint64(6)),
                  ('Sint64', Sint64(7)),
                  ('Real32', Real32(8)),
                  ('Real64', Real64(9)),
                  ('Bool', True),
                  ('DTN', CIMDateTime.now()),
                  # ('DTI', timedelta(60)),
                  ('Ref', obj_name)]

        self.test_recorder.stage_pywbem_args(method='InvokeMethod',
                                             MethodName='Blah',
                                             ObjectName=obj_name,
                                             Params=params)
        method_result_tuple = None
        method_exception = None
        self.test_recorder.stage_pywbem_result(method_result_tuple,
                                               method_exception)
        self.test_recorder.record_staged()

        # reload the yaml to test created values
        test_yaml = self.loadYamlFile()
        test_yaml = test_yaml[0]
        pywbem_request = test_yaml['pywbem_request']
        self.assertEqual(pywbem_request['url'], 'http://acme.com:80')
        operation = pywbem_request['operation']
        self.assertEqual(operation['pywbem_method'], 'InvokeMethod')
        self.assertEqual(operation['MethodName'], 'Blah')

        param_dict = dict(params)
        self.assertEqual(len(param_dict), 14)

        self.assertEqual(param_dict['StringParam'], 'Spotty')
        self.assertEqual(param_dict['Uint8'], 1)
        self.assertEqual(param_dict['Bool'], True)
        # test other parameters
        ref = param_dict['Ref']
        self.assertEqual(ref, obj_name)

    def test_get_instance(self):
        """
        Emulates call to getInstance to test parameter processing.
        Currently creates the pywbem_request component.
        """
        InstanceName = self.create_ciminstancename()

        self.test_recorder.reset()
        self.test_recorder.stage_pywbem_args(
            method='GetInstance',
            InstanceName=InstanceName,
            LocalOnly=True,
            IncludeQualifiers=True,
            IncludeClassOrigin=True,
            PropertyList=['propertyblah'])

        instance = None
        exc = None

        self.test_recorder.stage_pywbem_result(instance, exc)
        self.test_recorder.record_staged()
        test_yaml = self.loadYamlFile()
        test_yaml = test_yaml[0]
        pywbem_request = test_yaml['pywbem_request']
        self.assertEqual(pywbem_request['url'], 'http://acme.com:80')
        operation = pywbem_request['operation']
        self.assertEqual(operation['pywbem_method'], 'GetInstance')
        self.assertEqual(operation['LocalOnly'], True)
        self.assertEqual(operation['IncludeQualifiers'], True)
        self.assertEqual(operation['PropertyList'], ['propertyblah'])
        pywbem_response = test_yaml['pywbem_response']
        self.assertEqual(pywbem_response, {})

    def test_create_instance(self):
        """Test record of create instance"""

        NewInstance = self.create_ciminstance()

        self.test_recorder.reset()
        self.test_recorder.stage_pywbem_args(
            method='CreateInstance',
            NewInstance=NewInstance,
            namespace='cim/blah')

        exc = None
        obj_name = self.create_ciminstancename()
        self.test_recorder.stage_pywbem_result(obj_name, exc)
        self.test_recorder.record_staged()
        test_yaml = self.loadYamlFile()
        test_yaml = test_yaml[0]
        pywbem_request = test_yaml['pywbem_request']
        self.assertEqual(pywbem_request['url'], 'http://acme.com:80')
        operation = pywbem_request['operation']
        self.assertEqual(operation['pywbem_method'], 'CreateInstance')
        returned_new_inst = operation['NewInstance']
        self.assertEqual(returned_new_inst['classname'], NewInstance.classname)
        pd = returned_new_inst['properties']

        # compare all properties returned against original
        self.assertEqual(len(pd), len(NewInstance.properties))
        for pn, pv in pd.items():
            prop = CIMProperty(pn, pv['value'], type=pv['type'],
                               array_size=pv['array_size'],
                               propagated=pv['propagated'],
                               is_array=pv['is_array'],
                               reference_class=pv['reference_class'],
                               qualifiers=pv['qualifiers'],
                               embedded_object=pv['embedded_object'])

            self.assertEqual(NewInstance.properties[pn], prop,
                             'Property compare failed orig %s, recreated %s' %
                             (NewInstance.properties[pn], prop))

    def test_open_enumerateInstancePaths(self):
        """Emulate staging of enumerateInstancePaths call. Tests building
        yaml for this call. Used to test the generation of context, eos
        result info.
        """
        namespace = 'root/cimv2'
        self.test_recorder.reset(pull_op=True)
        self.test_recorder.stage_pywbem_args(
            method='OpenEnumerateInstancePaths',
            ClassName='CIM_BLAH',
            namespace=namespace,
            FilterQueryLanguage='WQL',
            FilterQuery='Property = 3',
            OperationTimeout=40,
            ContinueOnError=False,
            MaxObjectCount=100)

        exc = None
        result = []
        result_tuple = pull_path_result_tuple(result, True, None)
        self.test_recorder.stage_pywbem_result(result_tuple, exc)
        self.test_recorder.record_staged()

        test_yaml = self.loadYamlFile()
        test_yaml = test_yaml[0]

        pywbem_request = test_yaml['pywbem_request']
        self.assertEqual(pywbem_request['url'], 'http://acme.com:80')
        operation = pywbem_request['operation']
        self.assertEqual(operation['pywbem_method'],
                         'OpenEnumerateInstancePaths')
        self.assertEqual(operation['ClassName'], 'CIM_BLAH')
        self.assertEqual(operation['MaxObjectCount'], 100)
        self.assertEqual(operation['FilterQueryLanguage'], 'WQL')
        self.assertEqual(operation['FilterQuery'], 'Property = 3')
        self.assertEqual(operation['ContinueOnError'], False)
        self.assertEqual(operation['OperationTimeout'], 40)

        pywbem_response = test_yaml['pywbem_response']
        pull_result = pywbem_response['pullresult']
        self.assertEqual(pull_result['paths'], [])
        self.assertEqual(pull_result['eos'], True)
        self.assertEqual(pull_result['context'], None)


class BaseLogOperationRecorderTests(BaseRecorderTests):
    """
    Test the LogOperationRecorder functions. Creates log entries and
    uses testfixture to validate results
    """
    def recorder_setup(self, log_detail_level='min', max_log_entry_size=None):
        """Setup the recorder for a defined max output size"""
        PywbemLoggers.create_logger('ops', 'file',
                                    log_filename=TEST_OUTPUT_LOG,
                                    log_detail_level=log_detail_level)

        PywbemLoggers.create_logger('http', 'file',
                                    log_filename=TEST_OUTPUT_LOG,
                                    log_detail_level=log_detail_level)

        self.test_recorder = _LogOperationRecorder(max_log_entry_size)

        # Set a conn id into the connection. Saves testing the connection
        # log for each test.
        # pylint: disable=protected-access
        self.test_recorder._conn_id = 'test_id'
        self.test_recorder.reset()
        self.test_recorder.enable()

    def tearDown(self):
        """Remove LogCapture."""
        LogCapture.uninstall_all()
        logging.shutdown()
        # remove any existing log file
        if os.path.isfile(TEST_OUTPUT_LOG):
            os.remove(TEST_OUTPUT_LOG)


# Long log entry for getInstance return all log
get_inst_return_all_log = (
    u"Return:test_id GetInstance(CIMInstance(classname='CIM_Foo', path=None, "
    u"properties=NocaseDict({'Bool': CIMProperty(name='Bool', value=True, "
    u"type='boolean', reference_class=None, embedded_object=None, "
    u"is_array=False, array_size=None, class_origin=None, propagated=None, "
    u"qualifiers=NocaseDict({})), 'DTF': CIMProperty(name='DTF', "
    u"value=CIMDateTime(cimtype='datetime', '20160331193040.654321+120'), "
    u"type='datetime', reference_class=None, embedded_object=None, "
    u"is_array=False, array_size=None, class_origin=None, propagated=None, "
    u"qualifiers=NocaseDict({})), 'DTI': CIMProperty(name='DTI', "
    u"value=CIMDateTime(cimtype='datetime', '00000010000049.000020:000'), "
    u"type='datetime', reference_class=None, embedded_object=None, "
    u"is_array=False, array_size=None, class_origin=None, propagated=None, "
    u"qualifiers=NocaseDict({})), 'DTP': CIMProperty(name='DTP', "
    u"value=CIMDateTime(cimtype='datetime', '20140922104920.524789-399'), "
    u"type='datetime', reference_class=None, embedded_object=None, "
    u"is_array=False, array_size=None, class_origin=None, propagated=None, "
    u"qualifiers=NocaseDict({})), 'R32': CIMProperty(name='R32', "
    u"value=Real32(cimtype='real32', 42.0), type='real32', "
    u"reference_class=None, embedded_object=None, is_array=False, "
    u"array_size=None, class_origin=None, propagated=None, "
    u"qualifiers=NocaseDict({})), 'R64': CIMProperty(name='R64', "
    u"value=Real64(cimtype='real64', 42.64), type='real64', "
    u"reference_class=None, embedded_object=None, is_array=False, "
    u"array_size=None, class_origin=None, propagated=None, "
    u"qualifiers=NocaseDict({})), 'S1': CIMProperty(name='S1', value='Ham', "
    u"type='string', reference_class=None, embedded_object=None, "
    u"is_array=False, array_size=None, class_origin=None, propagated=None, "
    u"qualifiers=NocaseDict({})), 'S2': CIMProperty(name='S2', value='H\u00E4m'"
    u", type='string', reference_class=None, embedded_object=None, "
    u"is_array=False, array_size=None, class_origin=None, propagated=None, "
    u"qualifiers=NocaseDict({})), 'SI16': CIMProperty(name='SI16', "
    u"value=Sint16(cimtype='sint16', minvalue=-32768, maxvalue=32767, -4216), "
    u"type='sint16', reference_class=None, embedded_object=None, "
    u"is_array=False, array_size=None, class_origin=None, propagated=None, "
    u"qualifiers=NocaseDict({})), 'SI32': CIMProperty(name='SI32', "
    u"value=Sint32(cimtype='sint32', minvalue=-2147483648, "
    u"maxvalue=2147483647, -4232), type='sint32', reference_class=None, "
    u"embedded_object=None, is_array=False, array_size=None, "
    u"class_origin=None, propagated=None, qualifiers=NocaseDict({})), 'SI64': "
    u"CIMProperty(name='SI64', value=Sint64(cimtype='sint64', "
    u"minvalue=-9223372036854775808, maxvalue=9223372036854775807, -4264), "
    u"type='sint64', reference_class=None, embedded_object=None, "
    u"is_array=False, array_size=None, class_origin=None, propagated=None, "
    u"qualifiers=NocaseDict({})), 'SI8': CIMProperty(name='SI8', "
    u"value=Sint8(cimtype='sint8', minvalue=-128, maxvalue=127, -42), "
    u"type='sint8', reference_class=None, embedded_object=None, "
    u"is_array=False, array_size=None, class_origin=None, propagated=None, "
    u"qualifiers=NocaseDict({})), 'UI16': CIMProperty(name='UI16', "
    u"value=Uint16(cimtype='uint16', minvalue=0, maxvalue=65535, 4216), "
    u"type='uint16', reference_class=None, embedded_object=None, "
    u"is_array=False, array_size=None, class_origin=None, propagated=None, "
    u"qualifiers=NocaseDict({})), 'UI32': CIMProperty(name='UI32', "
    u"value=Uint32(cimtype='uint32', minvalue=0, maxvalue=4294967295, 4232), "
    u"type='uint32', reference_class=None, embedded_object=None, "
    u"is_array=False, array_size=None, class_origin=None, propagated=None, "
    u"qualifiers=NocaseDict({})), 'UI64': CIMProperty(name='UI64', "
    u"value=Uint64(cimtype='uint64', minvalue=0, "
    u"maxvalue=18446744073709551615, 4264), type='uint64', "
    u"reference_class=None, embedded_object=None, is_array=False, "
    u"array_size=None, class_origin=None, propagated=None, "
    u"qualifiers=NocaseDict({})), 'UI8': CIMProperty(name='UI8', "
    u"value=Uint8(cimtype='uint8', minvalue=0, maxvalue=255, 42), "
    u"type='uint8', reference_class=None, embedded_object=None, "
    u"is_array=False, array_size=None, class_origin=None, propagated=None, "
    u"qualifiers=NocaseDict({}))}), property_list=None, "
    u"qualifiers=NocaseDict({})))")


get_inst_return_all_log_PY2 = (
    "Return:test_id GetInstance(CIMInstance(classname=u'CIM_Foo', path=None, "
    "properties=NocaseDict({'Bool': CIMProperty(name=u'Bool', value=True, "
    "type='boolean', reference_class=None, embedded_object=None, is_array="
    "False, array_size=None, class_origin=None, propagated=None, qualifiers="
    "NocaseDict({})), 'DTF': CIMProperty(name=u'DTF', value=CIMDateTime("
    "cimtype='datetime', '20160331193040.654321+120'), type='datetime', "
    "reference_class=None, embedded_object=None, is_array=False, array_size="
    "None, class_origin=None, propagated=None, qualifiers=NocaseDict({})), "
    "'DTI': CIMProperty(name=u'DTI', value=CIMDateTime(cimtype='datetime', "
    "'00000010000049.000020:000'), type='datetime', reference_class=None, "
    "embedded_object=None, is_array=False, array_size=None, class_origin=None,"
    " propagated=None, qualifiers=NocaseDict({})), 'DTP': CIMProperty(name="
    "u'DTP', value=CIMDateTime(cimtype='datetime', '20140922104920.524789-399')"
    ", type='datetime', reference_class=None, embedded_object=None, is_array="
    "False, array_size=None, class_origin=None, propagated=None, qualifiers="
    "NocaseDict({})), 'R32': CIMProperty(name=u'R32', value=Real32(cimtype="
    "'real32', 42.0), type='real32', reference_class=None, embedded_object="
    "None, is_array=False, array_size=None, class_origin=None, propagated="
    "None, qualifiers=NocaseDict({})), 'R64': CIMProperty(name=u'R64', "
    "value=Real64(cimtype='real64', 42.64), type='real64', reference_class="
    "None, embedded_object=None, is_array=False, array_size=None, "
    "class_origin=None, propagated=None, qualifiers=NocaseDict({})), 'S1': "
    "CIMProperty(name=u'S1', value=u'Ham', type='string', reference_class=None"
    ", embedded_object=None, is_array=False, array_size=None, class_origin="
    "None, propagated=None, qualifiers=NocaseDict({})), 'SI16': CIMProperty("
    "name=u'SI16', value=Sint16(cimtype='sint16', minvalue=-32768, maxvalue="
    "32767, -4216), type='sint16', reference_class=None, embedded_object=None"
    ", is_array=False, array_size=None, class_origin=None, propagated=None, "
    "qualifiers=NocaseDict({})), 'SI32': CIMProperty(name=u'SI32', value="
    "Sint32(cimtype='sint32', minvalue=-2147483648, maxvalue=2147483647, -4232"
    "), type='sint32', reference_class=None, embedded_object=None, is_array="
    "False, array_size=None, class_origin=None, propagated=None, qualifiers="
    "NocaseDict({})), 'SI64': CIMProperty(name=u'SI64', value=Sint64(cimtype="
    "'sint64', minvalue=-9223372036854775808, maxvalue=9223372036854775807, "
    "-4264), type='sint64', reference_class=None, embedded_object=None, "
    "is_array=False, array_size=None, class_origin=None, propagated=None, "
    "qualifiers=NocaseDict({})), 'SI8': CIMProperty(name=u'SI8', value=Sint8("
    "cimtype='sint8', minvalue=-128, maxvalue=127, -42), type='sint8', "
    "reference_class=None, embedded_object=None, is_array=False, array_size="
    "None, class_origin=None, propagated=None, qualifiers=NocaseDict({})), "
    "'UI16': CIMProperty(name=u'UI16', value=Uint16(cimtype='uint16', minvalue"
    "=0, maxvalue=65535, 4216), type='uint16', reference_class=None, "
    "embedded_object=None, is_array=False, array_size=None, class_origin=None,"
    " propagated=None, qualifiers=NocaseDict({})), 'UI32': CIMProperty("
    "name=u'UI32', value=Uint32(cimtype='uint32', minvalue=0, maxvalue="
    "4294967295, 4232), type='uint32', reference_class=None, embedded_object"
    "=None, is_array=False, array_size=None, class_origin=None, propagated="
    "None, qualifiers=NocaseDict({})), 'UI64': CIMProperty(name=u'UI64', "
    "value=Uint64(cimtype='uint64', minvalue=0, maxvalue=18446744073709551615"
    ", 4264), type='uint64', reference_class=None, embedded_object=None, "
    "is_array=False, array_size=None, class_origin=None, propagated=None, "
    "qualifiers=NocaseDict({})), 'UI8': CIMProperty(name=u'UI8', "
    "value=Uint8(cimtype='uint8', minvalue=0, maxvalue=255, 42), "
    "type='uint8', reference_class=None, embedded_object=None, is_array=False"
    ", array_size=None, class_origin=None, propagated=None, qualifiers="
    "NocaseDict({}))}), property_list=None, qualifiers=NocaseDict({})))")


class LogOperationRecorderStagingTests(BaseLogOperationRecorderTests):
    """
    Test staging for different cim_operations.  This defines fixed
    parameters for the before and after staging, stages (which creates
    a yaml file), and then inspects that file to determine if valid
    yaml was created
    """
    @log_capture()
    def test_create_connection(self, lc):
        """Create connection with default parameters"""
        self.recorder_setup()
        # Fake the connection to create a fixed data environment
        conn = WBEMConnection('http://blah')
        conn.conn_id = '%s-%s' % (22, "1234:34")
        self.test_recorder.stage_wbem_connection(conn)

        lc.check(
            ("pywbem.ops", "DEBUG",
             "Connection:22-1234:34 WBEMConnection(url='http://blah', "
             "creds=None, conn_id=22-1234:34, "
             "default_namespace='root/cimv2', x509=None, verify_callback=None, "
             'ca_certs=None, no_verification=False, timeout=None, '
             'use_pull_operations=False, stats=False, '
             'recorders=[])'),)

    @log_capture()
    def test_create_connection2(self, lc):
        self.recorder_setup()

        x509_dict = {"cert_file": 'Certfile.x', 'key_file': 'keyfile.x'}
        conn = WBEMConnection('http://blah',
                              default_namespace='root/blah',
                              creds=('username', 'password'),
                              x509=x509_dict,
                              no_verification=True,
                              timeout=10,
                              use_pull_operations=True,
                              enable_stats=True)
        conn.conn_id = '%s-%s' % (23, "1234:34")
        self.test_recorder.stage_wbem_connection(conn)

        lc.check((
            "pywbem.ops", "DEBUG",
            "Connection:23-1234:34 WBEMConnection(url='http://blah', "
            "creds=('username', ...), conn_id=23-1234:34, "
            "default_namespace='root/blah', "
            "x509='cert_file': 'Certfile.x', 'key_file': 'keyfile.x', "
            "verify_callback=None, ca_certs=None, no_verification=True, "
            "timeout=10, use_pull_operations=True, stats=True, "
            "recorders=[])"),)

    @log_capture()
    def test_stage_result_exception(self, lc):
        """Test the ops result log None return, HTTPError exception."""
        self.recorder_setup(max_log_entry_size=10)
        ce = CIMError(6, "Fake CIMError")
        exc = HTTPError(500, "Fake Reason", cimerror='%s' % ce)
        self.test_recorder.stage_pywbem_result(None, exc)

        lc.check(
            ("pywbem.ops", "DEBUG",
             "Exception:test_id None('HTTPError...)"))

    @log_capture()
    def test_stage_result_exception_all(self, lc):
        """Test the ops result log None return, HTTPError exception."""
        self.recorder_setup(log_detail_level='all')
        ce = CIMError(6, "Fake CIMError")
        exc = HTTPError(500, "Fake Reason", cimerror='%s' % ce)
        self.test_recorder.stage_pywbem_result(None, exc)

        # TODO. V2 valid string has extra single quote after CIMError
        if six.PY2:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 "Exception:test_id None('HTTPError(500 (Fake Reason), "
                 "CIMError: 6: Fake CIMError)')"),)
        else:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 "Exception:test_id None('HTTPError(500 (Fake Reason), "
                 "CIMError: 6: Fake CIMError)')"),)

    @log_capture()
    def test_stage_getinstance_args(self, lc):
        """
        Emulates call to getInstance to test parameter processing.
        Currently creates the pywbem_request component.
        """

        inst_name = self.create_ciminstancename()

        self.recorder_setup(max_log_entry_size=10)

        self.test_recorder.stage_pywbem_args(
            method='GetInstance',
            InstanceName=inst_name,
            LocalOnly=True,
            IncludeQualifiers=True,
            IncludeClassOrigin=True,
            PropertyList=['propertyblah'])

        # pywbem 2 and 3 differ in only the use of unicode for certain
        # string properties. (ex. classname)
        if six.PY2:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 "Request:test_id GetInstance(IncludeClassOrigin=True, "
                 "IncludeQualifiers=True, InstanceName=CIMInstanceName("
                 "classname=u'CIM_Foo', keybindings=NocaseDict("
                 "{'Chicken': 'Ham'}), namespace=u'root/cimv2', "
                 "host=u'woot.com'), LocalOnly=True, "
                 "PropertyList=['propertyblah'])"),)

        else:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 'Request:test_id GetInstance(IncludeClassOrigin=True, '
                 'IncludeQualifiers=True, '
                 "InstanceName=CIMInstanceName(classname='CIM_Foo', "
                 "keybindings=NocaseDict({'Chicken': 'Ham'}), "
                 "namespace='root/cimv2', "
                 "host='woot.com'), LocalOnly=True, "
                 "PropertyList=['propertyblah'])"),)

    @log_capture()
    def test_stage_instance_result(self, lc):
        instance = self.create_ciminstance()
        self.recorder_setup(max_log_entry_size=10)
        exc = None
        self.test_recorder.stage_pywbem_result(instance, exc)

        if six.PY2:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 'Return:test_id None(CIMInstanc...)'))
        else:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 'Return:test_id None(CIMInstanc...)'))

    @log_capture()
    def test_stage_instance_result_default(self, lc):
        instance = self.create_ciminstance()
        self.recorder_setup()
        exc = None
        self.test_recorder.stage_pywbem_result(instance, exc)

        if six.PY2:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 "Return:test_id None(CIMInstance(classname=u'CIM_Foo', "
                 "path=None,"
                 " properties=NocaseDict({'Bool': CIMProperty(name=u'Bool', "
                 "value=True, type='boolean', reference_class=None, "
                 "embedded_object=None, is_array=False, array_size=None, "
                 "class_origin=None, propagated=None, "
                 "qualifiers=NocaseDict({})),"
                 " 'DTF': CIMProperty(name=u'DTF', value=CIMDateTime(cimtype"
                 "='datetime', '20160331193040.654321+120'), type='datetime', "
                 "reference_class=None, embedded_object=None, is_array=False, "
                 "array_size=None, class_origin=None, propagated=None, "
                 "qualifiers=NocaseDict({})), 'DTI': CIMProperty(name=u'DTI', "
                 "value=CIMDateTime(cimtype='datetime', "
                 "'00000010000049.000020:000'), type='datetime', "
                 "reference_class=None, embedded_object=None, is_array=False, "
                 "array_size=None, class_origin=None, propagated=None, "
                 "qualifiers=NocaseDict({})), 'DTP': CIMProperty(name=u'DTP', "
                 "value=CIMDateTime(cimtype='datetime', "
                 "'20140922104920.524789-399'), type='datetime', "
                 "reference_class=None, embedded_object=None, is_array=False, "
                 "array_size=None, class_origin=None, ...)"),)
        else:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 "Return:test_id None(CIMInstance(classname='CIM_Foo', "
                 "path=None, "
                 "properties=NocaseDict({'Bool': CIMProperty(name='Bool', "
                 "value=True, "
                 "type='boolean', reference_class=None, embedded_object=None, "
                 "is_array=False, array_size=None, class_origin=None, "
                 "propagated=None, "
                 "qualifiers=NocaseDict({})), 'DTF': CIMProperty(name='DTF', "
                 "value=CIMDateTime(cimtype='datetime', "
                 "'20160331193040.654321+120'), "
                 "type='datetime', reference_class=None, embedded_object=None, "
                 'is_array=False, array_size=None, class_origin=None, '
                 'propagated=None, '
                 "qualifiers=NocaseDict({})), 'DTI': CIMProperty(name='DTI', "
                 "value=CIMDateTime(cimtype='datetime', "
                 "'00000010000049.000020:000'), "
                 "type='datetime', reference_class=None, embedded_object=None, "
                 'is_array=False, array_size=None, class_origin=None, '
                 'propagated=None, '
                 "qualifiers=NocaseDict({})), 'DTP': CIMProperty(name='DTP', "
                 "value=CIMDateTime(cimtype='datetime', "
                 "'20140922104920.524789-399'), "
                 "type='datetime', reference_class=None, embedded_object=None, "
                 "is_array=False, array_size=None, class_origin=None, "
                 "propa...)"),)

    @log_capture()
    def test_stage_instance_result_all(self, lc):
        instance = self.create_ciminstance()
        self.recorder_setup(log_detail_level='all')
        exc = None
        self.test_recorder.stage_pywbem_result(instance, exc)

        if six.PY2:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 "Return:test_id None(CIMInstance(classname=u'CIM_Foo', "
                 "path=None, properties=NocaseDict({'Bool': CIMProperty("
                 "name=u'Bool', value=True, type='boolean', "
                 "reference_class=None, embedded_object=None, is_array=False,"
                 " array_size=None, class_origin=None, propagated=None, "
                 "qualifiers=NocaseDict({})), 'DTF': CIMProperty(name=u'DTF',"
                 " value=CIMDateTime(cimtype='datetime', "
                 "'20160331193040.654321+120'), type='datetime', "
                 "reference_class=None, embedded_object=None, is_array=False,"
                 " array_size=None, class_origin=None, propagated=None, "
                 "qualifiers=NocaseDict({})), 'DTI': CIMProperty(name=u'DTI',"
                 " value=CIMDateTime(cimtype='datetime', "
                 "'00000010000049.000020:000'), type='datetime', "
                 "reference_class=None, embedded_object=None, is_array=False,"
                 " array_size=None, class_origin=None, propagated=None, "
                 "qualifiers=NocaseDict({})), 'DTP': CIMProperty(name=u'DTP',"
                 " value=CIMDateTime(cimtype='datetime', "
                 "'20140922104920.524789-399'), type='datetime', "
                 "reference_class=None, embedded_object=None, is_array=False,"
                 " array_size=None, class_origin=None, propagated=None, "
                 "qualifiers=NocaseDict({})), 'R32': CIMProperty(name=u'R32',"
                 " value=Real32(cimtype='real32', 42.0), type='real32', "
                 "reference_class=None, embedded_object=None, is_array=False,"
                 " array_size=None, class_origin=None, propagated=None, "
                 "qualifiers=NocaseDict({})), 'R64': CIMProperty(name=u'R64',"
                 " value=Real64(cimtype='real64', 42.64), type='real64', "
                 "reference_class=None, embedded_object=None, is_array=False,"
                 " array_size=None, class_origin=None, propagated=None, "
                 "qualifiers=NocaseDict({})), 'S1': CIMProperty(name=u'S1',"
                 " value=u'Ham', type='string', reference_class=None, "
                 "embedded_object=None, is_array=False, array_size=None, "
                 "class_origin=None, propagated=None, qualifiers=NocaseDict("
                 "{})), 'SI16': CIMProperty(name=u'SI16', "
                 "value=Sint16(cimtype='sint16', minvalue=-32768,"
                 " maxvalue=32767, -4216), type='sint16', "
                 "reference_class=None, embedded_object=None, "
                 "is_array=False, array_size=None, class_origin=None, "
                 "propagated=None, qualifiers=NocaseDict({})), 'SI32': "
                 "CIMProperty(name=u'SI32', value=Sint32(cimtype='sint32', "
                 "minvalue=-2147483648, maxvalue=2147483647, -4232), "
                 "type='sint32', reference_class=None, embedded_object=None, "
                 "is_array=False, array_size=None, class_origin=None, "
                 "propagated=None, qualifiers=NocaseDict({})), 'SI64': "
                 "CIMProperty(name=u'SI64', value=Sint64(cimtype='sint64', "
                 "minvalue=-9223372036854775808, maxvalue=9223372036854775807,"
                 " -4264), type='sint64', reference_class=None, "
                 "embedded_object=None, is_array=False, array_size=None, "
                 "class_origin=None, propagated=None, qualifiers=NocaseDict("
                 "{})), 'SI8': CIMProperty(name=u'SI8', value=Sint8("
                 "cimtype='sint8', minvalue=-128, maxvalue=127, -42), "
                 "type='sint8', reference_class=None, embedded_object=None, "
                 "is_array=False, array_size=None, class_origin=None, "
                 "propagated=None, qualifiers=NocaseDict({})), 'UI16': "
                 "CIMProperty(name=u'UI16', value=Uint16(cimtype='uint16', "
                 "minvalue=0, maxvalue=65535, 4216), type='uint16', "
                 "reference_class=None, embedded_object=None, is_array=False,"
                 " array_size=None, class_origin=None, propagated=None, "
                 "qualifiers=NocaseDict({})), 'UI32': CIMProperty(name=u'UI32',"
                 " value=Uint32(cimtype='uint32', minvalue=0, "
                 "maxvalue=4294967295, 4232), type='uint32', reference_class="
                 "None, embedded_object=None, is_array=False, array_size=None,"
                 " class_origin=None, propagated=None, qualifiers=NocaseDict("
                 "{})), 'UI64': CIMProperty(name=u'UI64', value=Uint64(cimtype"
                 "='uint64', minvalue=0, maxvalue=18446744073709551615, 4264), "
                 "type='uint64', reference_class=None, embedded_object=None, "
                 "is_array=False, array_size=None, class_origin=None, "
                 "propagated=None, qualifiers=NocaseDict({})), 'UI8': "
                 "CIMProperty(name=u'UI8', value=Uint8(cimtype='uint8', "
                 "minvalue=0, maxvalue=255, 42), type='uint8', reference_"
                 "class=None, embedded_object=None, is_array=False, "
                 "array_size=None, class_origin=None, propagated=None, "
                 "qualifiers=NocaseDict({}))}), property_list=None, "
                 "qualifiers=NocaseDict({})))"),)
        else:
            none_result_all = get_inst_return_all_log.replace('GetInstance',
                                                              'None')
            lc.check(
                ("pywbem.ops", "DEBUG", none_result_all))


class LogOperationRecorderTests(BaseLogOperationRecorderTests):
    """Test args and resutls logging"""

    @log_capture()
    def test_getinstance(self, lc):
        """Test the ops result log for get instance"""

        inst_name = self.create_ciminstancename()

        # set recorder to limit response to length of 10
        self.recorder_setup(max_log_entry_size=10)

        self.test_recorder.stage_pywbem_args(
            method='GetInstance',
            InstanceName=inst_name,
            LocalOnly=True,
            IncludeQualifiers=True,
            IncludeClassOrigin=True,
            PropertyList=['propertyblah'])
        instance = self.create_ciminstance()
        exc = None
        self.test_recorder.stage_pywbem_result(instance, exc)

        if six.PY2:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 "Request:test_id GetInstance(IncludeClassOrigin=True, "
                 "IncludeQualifiers=True, InstanceName=CIMInstanceName("
                 "classname=u'CIM_Foo', keybindings=NocaseDict({'Chicken': "
                 "'Ham'}), namespace=u'root/cimv2', host=u'woot.com'), "
                 "LocalOnly=True, PropertyList=['propertyblah'])"),
                ('pywbem.ops', 'DEBUG',
                 'Return:test_id GetInstance(CIMInstanc...)'))
        else:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 'Request:test_id GetInstance(IncludeClassOrigin=True, '
                 'IncludeQualifiers=True, '
                 "InstanceName=CIMInstanceName(classname='CIM_Foo', "
                 "keybindings=NocaseDict({'Chicken': 'Ham'}), "
                 "namespace='root/cimv2', "
                 "host='woot.com'), LocalOnly=True, "
                 "PropertyList=['propertyblah'])"),
                ('pywbem.ops', 'DEBUG',
                 'Return:test_id GetInstance(CIMInstanc...)'))

    @log_capture()
    def test_getinstance_exception(self, lc):
        """Test the ops result log for get instance"""

        inst_name = self.create_ciminstancename()

        self.recorder_setup(max_log_entry_size=11)

        self.test_recorder.stage_pywbem_args(
            method='GetInstance',
            InstanceName=inst_name,
            LocalOnly=True,
            IncludeQualifiers=True,
            IncludeClassOrigin=True,
            PropertyList=['propertyblah'])
        instance = None
        exc = CIMError(6, "Fake CIMError")
        self.test_recorder.stage_pywbem_result(instance, exc)

        if six.PY2:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 "Request:test_id GetInstance(IncludeClassOrigin=True, "
                 "IncludeQualifiers=True, InstanceName=CIMInstanceName("
                 "classname=u'CIM_Foo', keybindings=NocaseDict({'Chicken': "
                 "'Ham'}), namespace=u'root/cimv2', host=u'woot.com'), "
                 "LocalOnly=True, PropertyList=['propertyblah'])"),
                ("pywbem.ops", "DEBUG",
                 "Exception:test_id GetInstance('CIMError(6...)"))
        else:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 "Request:test_id GetInstance(IncludeClassOrigin=True, "
                 "IncludeQualifiers=True, InstanceName=CIMInstanceName("
                 "classname='CIM_Foo', keybindings=NocaseDict({'Chicken': "
                 "'Ham'}), namespace='root/cimv2', host='woot.com'), "
                 "LocalOnly=True, PropertyList=['propertyblah'])"),
                ("pywbem.ops", "DEBUG",
                 "Exception:test_id GetInstance('CIMError(6...)"))

    @log_capture()
    def test_getinstance_exception_all(self, lc):
        """Test the ops result log for get instance CIMError exception"""

        inst_name = self.create_ciminstancename()

        self.recorder_setup(log_detail_level='all')

        self.test_recorder.stage_pywbem_args(
            method='GetInstance',
            InstanceName=inst_name)
        instance = None
        exc = CIMError(6, "Fake CIMError")
        self.test_recorder.stage_pywbem_result(instance, exc)

        if six.PY2:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 "Request:test_id GetInstance(InstanceName=CIMInstanceName("
                 "classname=u'CIM_Foo', keybindings=NocaseDict({'Chicken': "
                 "'Ham'}), namespace=u'root/cimv2', host=u'woot.com'))"),
                ("pywbem.ops", "DEBUG",
                 "Exception:test_id GetInstance('CIMError(6: Fake "
                 "CIMError)')"),)
        else:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 "Request:test_id GetInstance(InstanceName=CIMInstanceName("
                 "classname='CIM_Foo', keybindings=NocaseDict({'Chicken': "
                 "'Ham'}), namespace='root/cimv2', host='woot.com'))"),
                ("pywbem.ops", "DEBUG",
                 "Exception:test_id GetInstance('CIMError(6: Fake "
                 "CIMError)')"),)

    @log_capture()
    def test_getinstance_result_all(self, lc):
        """Test the ops result log for get instance"""

        inst_name = self.create_ciminstancename()

        self.recorder_setup(log_detail_level='all')

        self.test_recorder.stage_pywbem_args(
            method='GetInstance',
            InstanceName=inst_name,
            LocalOnly=True,
            IncludeQualifiers=True,
            IncludeClassOrigin=True,
            PropertyList=['propertyblah'])
        instance = self.create_ciminstance()
        exc = None
        self.test_recorder.stage_pywbem_result(instance, exc)

        if six.PY2:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 "Request:test_id GetInstance(IncludeClassOrigin=True, "
                 "IncludeQualifiers=True, InstanceName=CIMInstanceName("
                 "classname=u'CIM_Foo', keybindings=NocaseDict({'Chicken': "
                 "'Ham'}), namespace=u'root/cimv2', host=u'woot.com'), "
                 "LocalOnly=True, PropertyList=['propertyblah'])"),
                ('pywbem.ops',
                 'DEBUG',
                 get_inst_return_all_log_PY2))
        else:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 'Request:test_id GetInstance(IncludeClassOrigin=True, '
                 'IncludeQualifiers=True, '
                 "InstanceName=CIMInstanceName(classname='CIM_Foo', "
                 "keybindings=NocaseDict({'Chicken': 'Ham'}), "
                 "namespace='root/cimv2', "
                 "host='woot.com'), LocalOnly=True, "
                 "PropertyList=['propertyblah'])"),
                ("pywbem.ops", "DEBUG", get_inst_return_all_log),)

    @log_capture()
    def test_enuminstances_result(self, lc):
        """Test the ops result log for enumerate instances"""

        # set recorder to limit response to length of 10
        self.recorder_setup(max_log_entry_size=10)

        self.test_recorder.stage_pywbem_args(
            method='EnumerateInstances',
            ClassName='CIM_Foo',
            LocalOnly=True,
            IncludeQualifiers=True,
            IncludeClassOrigin=True,
            PropertyList=['propertyblah'])

        instance = self.create_ciminstance()
        exc = None
        self.test_recorder.stage_pywbem_result([instance, instance], exc)

        if six.PY2:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 "Request:test_id EnumerateInstances(ClassName='CIM_Foo', "
                 "IncludeClassOrigin=True, IncludeQualifiers=True, "
                 "LocalOnly=True, PropertyList=['propertyblah'])"),
                ('pywbem.ops', 'DEBUG',
                 'Return:test_id EnumerateInstances([CIMInstan...)'))
        else:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 "Request:test_id EnumerateInstances(ClassName='CIM_Foo', "
                 'IncludeClassOrigin=True, IncludeQualifiers=True, '
                 'LocalOnly=True, '
                 "PropertyList=['propertyblah'])"),
                ('pywbem.ops', 'DEBUG',
                 'Return:test_id EnumerateInstances([CIMInstan...)'))

    @log_capture()
    def test_enuminstancenames_result(self, lc):
        """Test the ops result log for enumerate instances"""

        # set recorder to limit response to length of 10
        self.recorder_setup(max_log_entry_size=10)

        self.test_recorder.stage_pywbem_args(
            method='EnumerateInstanceNames',
            ClassName='CIM_Foo',
            LocalOnly=True,
            IncludeQualifiers=True,
            IncludeClassOrigin=True,
            PropertyList=['propertyblah', 'blah2'])

        exc = None
        inst_name = self.create_ciminstancename()
        self.test_recorder.stage_pywbem_result([inst_name, inst_name], exc)

        if six.PY2:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 "Request:test_id EnumerateInstanceNames(ClassName='CIM_Foo', "
                 "IncludeClassOrigin=True, IncludeQualifiers=True, "
                 "LocalOnly=True, PropertyList=['propertyblah', 'blah2'])"),
                ('pywbem.ops', 'DEBUG',
                 'Return:test_id EnumerateInstanceNames([CIMInstan...)'))
        else:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 "Request:test_id EnumerateInstanceNames(ClassName='CIM_Foo', "
                 'IncludeClassOrigin=True, IncludeQualifiers=True, '
                 'LocalOnly=True, '
                 "PropertyList=['propertyblah', 'blah2'])"),
                ('pywbem.ops', 'DEBUG',
                 'Return:test_id EnumerateInstanceNames([CIMInstan...)'))

    @log_capture()
    def test_openenuminstances_result_all(self, lc):
        """Test the ops result log for enumerate instances. Returns no
        instances.
        """

        # set recorder to limit response to length of 10
        self.recorder_setup(log_detail_level='all')

        self.test_recorder.stage_pywbem_args(
            method='OpenEnumerateInstances',
            ClassName='CIM_Foo',
            LocalOnly=True,
            IncludeQualifiers=True,
            IncludeClassOrigin=True,
            PropertyList=['propertyblah'])

        # instance = self.create_ciminstance()
        exc = None

        result = []
        context = ('test_rtn_context', 'root/blah')
        result_tuple = pull_inst_result_tuple(result, False, context)

        self.test_recorder.stage_pywbem_result(result_tuple, exc)

        lc.check(
            ("pywbem.ops", "DEBUG",
             "Request:test_id OpenEnumerateInstances(ClassName='CIM_Foo', "
             "IncludeClassOrigin=True, IncludeQualifiers=True, "
             "LocalOnly=True, PropertyList=['propertyblah'])"),
            ('pywbem.ops', 'DEBUG',
             "Return:test_id OpenEnumerateInstances(pull_inst_result_tuple("
             "context=('test_rtn_context', 'root/blah'), eos=False, "
             "instances=[]))"),)

    @log_capture()
    def test_openenuminstancepaths_result_all(self, lc):
        """Test the ops result log for enumerate instances paths with
        data in the paths component"""

        # set recorder to limit response to length of 10
        self.recorder_setup(log_detail_level='all')

        self.test_recorder.stage_pywbem_args(
            method='OpenEnumerateInstancePaths',
            ClassName='CIM_Foo',
            FilterQueryLanguage='FQL',
            FilterQuery='SELECT A from B',
            OperationTimeout=10,
            ContinueOnError=None,
            MaxObjectCount=100)

        inst_name = self.create_ciminstancename()
        result = [inst_name, inst_name]
        exc = None

        context = ('test_rtn_context', 'root/blah')
        result_tuple = pull_path_result_tuple(result, False, context)

        self.test_recorder.stage_pywbem_result(result_tuple, exc)

        if six.PY2:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 "Request:test_id OpenEnumerateInstancePaths(ClassName="
                 "'CIM_Foo', ContinueOnError=None, FilterQuery='SELECT A "
                 "from B', FilterQueryLanguage='FQL', MaxObjectCount=100, "
                 "OperationTimeout=10)"),
                ('pywbem.ops', 'DEBUG',
                 "Return:test_id OpenEnumerateInstancePaths("
                 "pull_path_result_tuple(context=('test_rtn_context', "
                 "'root/blah'), eos=False, paths=[CIMInstanceName("
                 "classname=u'CIM_Foo', keybindings=NocaseDict({'Chicken': "
                 "'Ham'}), namespace=u'root/cimv2', host=u'woot.com'), "
                 "CIMInstanceName(classname=u'CIM_Foo', keybindings="
                 "NocaseDict({'Chicken': 'Ham'}), namespace=u'root/cimv2',"
                 " host=u'woot.com')]))"))

        else:
            lc.check(
                ("pywbem.ops", "DEBUG",
                 "Request:test_id OpenEnumerateInstancePaths("
                 "ClassName='CIM_Foo', "
                 "ContinueOnError=None, FilterQuery='SELECT A from B', "
                 "FilterQueryLanguage='FQL', MaxObjectCount=100, "
                 "OperationTimeout=10)"),
                ('pywbem.ops', 'DEBUG',
                 'Return:test_id '
                 "OpenEnumerateInstancePaths(pull_path_result_tuple("
                 "context=('test_rtn_context', "
                 "'root/blah'), eos=False, paths=[CIMInstanceName("
                 "classname='CIM_Foo', "
                 "keybindings=NocaseDict({'Chicken': 'Ham'}), "
                 "namespace='root/cimv2', "
                 "host='woot.com'), CIMInstanceName(classname='CIM_Foo', "
                 "keybindings=NocaseDict({'Chicken': 'Ham'}), "
                 "namespace='root/cimv2', "
                 "host='woot.com')]))"),)

    @log_capture()
    def test_associators_result(self, lc):
        """Test the ops result log for Associators that returns nothing"""

        inst_name = self.create_ciminstancename()

        # set recorder to limit response to length of 10
        self.recorder_setup(max_log_entry_size=10)

        self.test_recorder.stage_pywbem_args(
            method='Associators',
            InstanceName=inst_name,
            AssocClass='BLAH_Assoc',
            ResultClass='BLAH_Result',
            IncludeQualifiers=True,
            IncludeClassOrigin=True,
            PropertyList=['propertyblah', 'propertyblah2'])
        exc = None
        self.test_recorder.stage_pywbem_result([], exc)

        if six.PY2:
            lc.check(
                ('pywbem.ops', 'DEBUG',
                 "Request:test_id Associators(AssocClass='BLAH_Assoc', "
                 "IncludeClassOrigin=True, IncludeQualifiers=True, "
                 "InstanceName=CIMInstanceName(classname=u'CIM_Foo', "
                 "keybindings=NocaseDict({'Chicken': 'Ham'}), "
                 "namespace=u'root/cimv2', host=u'woot.com'), "
                 "PropertyList=['propertyblah', 'propertyblah2'], "
                 "ResultClass='BLAH_Result')"),
                ('pywbem.ops', 'DEBUG', 'Return:test_id Associators([])'))

        else:
            lc.check(
                ('pywbem.ops', 'DEBUG',
                 "Request:test_id Associators(AssocClass='BLAH_Assoc', "
                 'IncludeClassOrigin=True, IncludeQualifiers=True, '
                 "InstanceName=CIMInstanceName(classname='CIM_Foo', "
                 "keybindings=NocaseDict({'Chicken': 'Ham'}), "
                 "namespace='root/cimv2', "
                 "host='woot.com'), PropertyList=['propertyblah', "
                 "'propertyblah2'], "
                 "ResultClass='BLAH_Result')"),
                ('pywbem.ops', 'DEBUG', 'Return:test_id Associators([])'))

    @log_capture()
    def test_associators_result_exception(self, lc):
        """Test the ops result log for associators that returns exception"""

        # set recorder to limit response to length of 10
        self.recorder_setup(max_log_entry_size=11)

        exc = CIMError(6, "Fake CIMError")
        self.test_recorder.stage_pywbem_result([], exc)

        lc.check(
            ('pywbem.ops', 'DEBUG',
             "Exception:test_id None('CIMError(6...)"),)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

"""
Exercise routines in cim_xml by creating xml document fragments and
passing them through a validator.
"""
#
# TODO: A bunch of tests are still unimplemented for bits of the
# schema that PyWBEM doesn't use right now.
#

from __future__ import absolute_import

import sys
import unittest

from pywbem import cim_xml
from pywbem.cim_obj import _ensure_bytes

DTD_FILE = 'CIM_DTD_V22.dtd'


def validate_xml(data, dtd_directory=None):
    """Run xmllint to validate files in dtd_directory"""

    from subprocess import Popen, PIPE

    # Run xmllint to validate file

    dtd_file = DTD_FILE
    if dtd_directory is not None:
        dtd_file = '%s/%s' % (dtd_directory, DTD_FILE)

    p = Popen('xmllint --dtdvalid %s --noout -' % dtd_file, stdout=PIPE,
              stdin=PIPE, stderr=PIPE, shell=True)

    data = _ensure_bytes(data)
    p.stdin.write(data)
    p.stdin.close()

    for x in p.stdout.readlines():
        sys.stdout.write(x)

    status = p.wait()

    if status != 0:
        return False

    return True

# Test data to save typing


def LOCALNAMESPACEPATH():   # pylint: disable=invalid-name
    """Return a LOCALNAMESPACEPATH"""
    return cim_xml.LOCALNAMESPACEPATH([cim_xml.NAMESPACE('root'),
                                       cim_xml.NAMESPACE('cimv2')])


def NAMESPACEPATH():   # pylint: disable=invalid-name
    return cim_xml.NAMESPACEPATH(
        cim_xml.HOST('leonardo'), LOCALNAMESPACEPATH())


def CLASSNAME():  # pylint: disable=invalid-name
    return cim_xml.CLASSNAME('CIM_Foo')


def INSTANCENAME():  # pylint: disable=invalid-name
    return cim_xml.INSTANCENAME(
        'CIM_Pet',
        [cim_xml.KEYBINDING('type', cim_xml.KEYVALUE('dog', 'string')),
         cim_xml.KEYBINDING('age', cim_xml.KEYVALUE('2', 'numeric'))])

# Base classes


class CIMXMLTest(unittest.TestCase):
    """Run validate.py script against an xml document fragment."""

    def setUp(self):

        # List of test cases, each list item being an xml.dom.minidom node
        # representing some element from the CIM-XML payload.
        self.xml = []

        # List of expected XML strings resulting from each test case.
        self.xml_str = []

    @staticmethod
    def validate(xml, expectedResult=0):
        # pylint: disable=unused-argument
        """xml is a string with the CIM-XML."""
        validate_xml(xml, dtd_directory='../..')

    def test_all(self):

        for i in range(0, len(self.xml)):

            xml_node = self.xml[i]
            act_xml_str = xml_node.toxml()

            # Test that the XML fragments pass validation against CIM-XML DTD
            self.validate(act_xml_str)

            if i < len(self.xml_str):
                # Test XML fragments for expected string representation
                exp_xml_str = self.xml_str[i]
                if exp_xml_str is not None:
                    self.assertEqual(act_xml_str, exp_xml_str,
                                     "CIM-XML fragment to be tested: %r" %
                                     act_xml_str)


# pylint: disable=too-few-public-methods
class UnimplementedTest(object):

    @staticmethod
    def test_all():
        raise AssertionError('Unimplemented test')


#################################################################
#     3.2.1. Top Level Elements
#################################################################

#     3.2.1.1. CIM


class CIM(CIMXMLTest):
    def setUp(self):
        super(CIM, self).setUp()
        self.xml.append(cim_xml.CIM(
            cim_xml.MESSAGE(
                cim_xml.SIMPLEREQ(
                    cim_xml.IMETHODCALL(
                        'IntrinsicMethod',
                        LOCALNAMESPACEPATH())),
                '1001', '1.0'),
            '2.0', '2.0'))


#################################################################
#     3.2.2. Declaration Elements
#################################################################

#     3.2.2.1. DECLARATION
#     3.2.2.2. DECLGROUP
#     3.2.2.3. DECLGROUP.WITHNAME
#     3.2.2.4. DECLGROUP.WITHPATH
#     3.2.2.5. QUALIFIER.DECLARATION
#     3.2.2.6. SCOPE


# pylint: disable=too-few-public-methods
class Declaration(UnimplementedTest):
    """
    <!ELEMENT DECLARATION  (DECLGROUP|DECLGROUP.WITHNAME|DECLGROUP.WITHPATH)+>
    """


# pylint: disable=too-few-public-methods
class DeclGroup(UnimplementedTest):
    """
    <!ELEMENT DECLGROUP  ((LOCALNAMESPACEPATH|NAMESPACEPATH)?,
                          QUALIFIER.DECLARATION*,VALUE.OBJECT*)>
    """
    pass


# pylint: disable=too-few-public-methods
class DeclGroupWithName(UnimplementedTest):
    """
    <!ELEMENT DECLGROUP.WITHNAME  ((LOCALNAMESPACEPATH|NAMESPACEPATH)?,
                                   QUALIFIER.DECLARATION*,VALUE.NAMEDOBJECT*)>
    """


# pylint: disable=too-few-public-methods
class DeclGroupWithPath(UnimplementedTest):
    """
    <!ELEMENT DECLGROUP.WITHPATH  (VALUE.OBJECTWITHPATH|
                                   VALUE.OBJECTWITHLOCALPATH)*>
    """


# pylint: disable=too-few-public-methods
class QualifierDeclaration(UnimplementedTest):
    """
    <!ELEMENT QUALIFIER.DECLARATION (SCOPE?, (VALUE | VALUE.ARRAY)?)>
    <!ATTLIST QUALIFIER.DECLARATION
        %CIMName;
        %CIMType;               #REQUIRED
        ISARRAY    (true|false) #IMPLIED
        %ArraySize;
        %QualifierFlavor;>
    """


class Scope(CIMXMLTest):
    """
    <!ELEMENT SCOPE EMPTY>
    <!ATTLIST SCOPE
         CLASS        (true|false)      'false'
         ASSOCIATION  (true|false)      'false'
         REFERENCE    (true|false)      'false'
         PROPERTY     (true|false)      'false'
         METHOD       (true|false)      'false'
         PARAMETER    (true|false)      'false'
         INDICATION   (true|false)      'false'>
    """

    def setUp(self):
        super(Scope, self).setUp()
        self.xml.append(cim_xml.SCOPE())


#################################################################
#     3.2.3. Value Elements
#################################################################

#     3.2.3.1. VALUE
#     3.2.3.2. VALUE.ARRAY
#     3.2.3.3. VALUE.REFERENCE
#     3.2.3.4. VALUE.REFARRAY
#     3.2.3.5. VALUE.OBJECT
#     3.2.3.6. VALUE.NAMEDINSTANCE
#     3.2.3.7. VALUE.NAMEDOBJECT
#     3.2.3.8. VALUE.OBJECTWITHPATH
#     3.2.3.9. VALUE.OBJECTWITHLOCALPATH
#     3.2.3.10. VALUE.NULL


class Value(CIMXMLTest):
    """
    <!ELEMENT VALUE (#PCDATA)>
    """

    def setUp(self):
        super(Value, self).setUp()

        # The VALUE element depends on whether XML-based or CDATA-based
        # escaping is used. Therefore, we modify the module-level switch that
        # controls that and run each test twice (wth different expected XML
        # strings).

        cim_xml._CDATA_ESCAPING = True  # pylint: disable=protected-access

        self.xml.append(cim_xml.VALUE('dog'))
        self.xml_str.append('<VALUE>dog</VALUE>')

        # self.xml.append(cim_xml.VALUE(None))
        # Note: This is illegal, Value.Null should be used instead.

        self.xml.append(cim_xml.VALUE(''))
        self.xml_str.append('<VALUE></VALUE>')  # Assum. not folded to <VALUE/>

        # Some control characters
        self.xml.append(cim_xml.VALUE('a\nb\rc\td'))
        self.xml_str.append('<VALUE>a\nb\rc\td</VALUE>')  # Assuming XML 1.1

        # Some XML special characters
        self.xml.append(cim_xml.VALUE('a&b<c>d'))
        self.xml_str.append('<VALUE><![CDATA[a&b<c>d]]></VALUE>')

        # Some XML special characters, already XML-escaped
        self.xml.append(cim_xml.VALUE('a&amp;b&lt;c&gt;d'))
        self.xml_str.append('<VALUE><![CDATA[a&amp;b&lt;c&gt;d]]></VALUE>')

        # Some XML special characters, already CDATA-escaped
        self.xml.append(cim_xml.VALUE('<![CDATA[a&b<c>d]]>'))
        self.xml_str.append(
            '<VALUE><![CDATA[<![CDATA[a&b<c>d]]]><![CDATA[]>]]></VALUE>')

        # set back to its default
        cim_xml._CDATA_ESCAPING = False  # pylint: disable=protected-access

        self.xml.append(cim_xml.VALUE('dog'))
        self.xml_str.append('<VALUE>dog</VALUE>')

        # self.xml.append(cim_xml.VALUE(None))
        # Note: This is illegal, Value.Null is used instead.

        self.xml.append(cim_xml.VALUE(''))
        self.xml_str.append('<VALUE></VALUE>')  # Assum. not folded to <VALUE/>

        # Some control characters
        self.xml.append(cim_xml.VALUE('a\nb\rc\td'))
        self.xml_str.append('<VALUE>a\nb\rc\td</VALUE>')  # Assuming XML 1.1

        # Some XML special characters
        self.xml.append(cim_xml.VALUE('a&b<c>d'))
        self.xml_str.append('<VALUE>a&amp;b&lt;c&gt;d</VALUE>')

        # Some XML special characters, already XML-escaped
        self.xml.append(cim_xml.VALUE('a&amp;b&lt;c&gt;d'))
        self.xml_str.append('<VALUE>a&amp;amp;b&amp;lt;c&amp;gt;d</VALUE>')

        # Some XML special characters, already CDATA-escaped
        self.xml.append(cim_xml.VALUE('<![CDATA[a&b<c>d]]>'))
        self.xml_str.append(
            '<VALUE>&lt;![CDATA[a&amp;b&lt;c&gt;d]]&gt;</VALUE>')


class ValueArray(CIMXMLTest):
    """
    <!ELEMENT VALUE.ARRAY (VALUE*)>
    """

    def setUp(self):
        super(ValueArray, self).setUp()

        self.xml.append(cim_xml.VALUE_ARRAY([]))

        self.xml.append(cim_xml.VALUE_ARRAY([cim_xml.VALUE('cat'),
                                             cim_xml.VALUE('dog')]))


class ValueReference(CIMXMLTest):
    """
    <!ELEMENT VALUE.REFERENCE (CLASSPATH|LOCALCLASSPATH|CLASSNAME|
                               INSTANCEPATH|LOCALINSTANCEPATH|INSTANCENAME)>
    """

    def setUp(self):
        super(ValueReference, self).setUp()

        # CLASSPATH

        self.xml.append(cim_xml.VALUE_REFERENCE(
            cim_xml.CLASSPATH(NAMESPACEPATH(), CLASSNAME())))

        # LOCALCLASSPATH

        self.xml.append(cim_xml.VALUE_REFERENCE(
            cim_xml.LOCALCLASSPATH(LOCALNAMESPACEPATH(), CLASSNAME())))

        # CLASSNAME

        self.xml.append(cim_xml.VALUE_REFERENCE(CLASSNAME()))

        # INSTANCEPATH

        self.xml.append(cim_xml.VALUE_REFERENCE(
            cim_xml.INSTANCEPATH(
                NAMESPACEPATH(), INSTANCENAME())))

        # LOCALINSTANCEPATH

        self.xml.append(cim_xml.VALUE_REFERENCE(
            cim_xml.LOCALINSTANCEPATH(
                LOCALNAMESPACEPATH(), INSTANCENAME())))

        # INSTANCENAME

        self.xml.append(cim_xml.VALUE_REFERENCE(INSTANCENAME()))


class ValueRefArray(CIMXMLTest):
    """
    <!ELEMENT VALUE.REFARRAY (VALUE.REFERENCE*)>
    """

    def setUp(self):
        super(ValueRefArray, self).setUp()

        # Empty

        self.xml.append(cim_xml.VALUE_REFARRAY([]))

        # VALUE.REFARRAY

        self.xml.append(cim_xml.VALUE_REFARRAY(
            [cim_xml.VALUE_REFERENCE(cim_xml.CLASSNAME('CIM_Foo')),
             cim_xml.VALUE_REFERENCE(cim_xml.LOCALCLASSPATH(
                 LOCALNAMESPACEPATH(), CLASSNAME()))]))


class ValueObject(CIMXMLTest):
    """
    <!ELEMENT VALUE.OBJECT (CLASS|INSTANCE)>
    """

    def setUp(self):
        super(ValueObject, self).setUp()

        # CLASS

        self.xml.append(cim_xml.VALUE_OBJECT(cim_xml.CLASS('CIM_Foo')))

        # INSTANCE

        self.xml.append(cim_xml.VALUE_OBJECT(cim_xml.INSTANCE('CIM_Pet', [])))


class ValueNamedInstance(CIMXMLTest):
    """
    <!ELEMENT VALUE.NAMEDINSTANCE (INSTANCENAME,INSTANCE)>
    """

    def setUp(self):
        super(ValueNamedInstance, self).setUp()

        self.xml.append(cim_xml.VALUE_NAMEDINSTANCE(
            INSTANCENAME(),
            cim_xml.INSTANCE('CIM_Pet', [])))


class ValueNamedObject(CIMXMLTest):
    """
    <!ELEMENT VALUE.NAMEDOBJECT (CLASS|(INSTANCENAME,INSTANCE))>
    """

    def setUp(self):
        super(ValueNamedObject, self).setUp()

        # CLASS

        self.xml.append(cim_xml.VALUE_NAMEDOBJECT(
            cim_xml.CLASS('CIM_Foo')))

        # INSTANCENAME, INSTANCE

        self.xml.append(cim_xml.VALUE_NAMEDOBJECT(
            (INSTANCENAME(),
             cim_xml.INSTANCE('CIM_Pet', []))))


class ValueObjectWithPath(CIMXMLTest):
    """
    <!ELEMENT VALUE.OBJECTWITHPATH ((CLASSPATH,CLASS)|
                                    (INSTANCEPATH,INSTANCE))>
    """

    def setUp(self):
        super(ValueObjectWithPath, self).setUp()

        # (CLASSPATH, CLASS)

        self.xml.append(cim_xml.VALUE_OBJECTWITHPATH(
            cim_xml.CLASSPATH(NAMESPACEPATH(), CLASSNAME()),
            cim_xml.CLASS('CIM_Foo')))

        # (INSTANCEPATH, INSTANCE)

        self.xml.append(cim_xml.VALUE_OBJECTWITHPATH(
            cim_xml.INSTANCEPATH(
                NAMESPACEPATH(), INSTANCENAME()),
            cim_xml.INSTANCE('CIM_Pet', [])))


class ValueObjectWithLocalPath(CIMXMLTest):
    """
    <!ELEMENT VALUE.OBJECTWITHLOCALPATH ((LOCALCLASSPATH,CLASS)|
                                         (LOCALINSTANCEPATH,INSTANCE))>
    """

    def setUp(self):
        super(ValueObjectWithLocalPath, self).setUp()

        # (LOCALCLASSPATH, CLASS)

        self.xml.append(cim_xml.VALUE_OBJECTWITHLOCALPATH(
            cim_xml.LOCALCLASSPATH(LOCALNAMESPACEPATH(), CLASSNAME()),
            cim_xml.CLASS('CIM_Foo')))

        # (LOCALINSTANCEPATH, INSTANCE)

        self.xml.append(cim_xml.VALUE_OBJECTWITHLOCALPATH(
            cim_xml.LOCALINSTANCEPATH(
                LOCALNAMESPACEPATH(),
                INSTANCENAME()),
            cim_xml.INSTANCE('CIM_Pet', [])))


# pylint: disable=too-few-public-methods
class ValueNull(UnimplementedTest):
    """
    <!ELEMENT VALUE.NULL EMPTY>
    """
    # TODO: Implement ValueNull test


#################################################################
#     3.2.4. Naming and Location Elements
#################################################################

#     3.2.4.1. NAMESPACEPATH
#     3.2.4.2. LOCALNAMESPACEPATH
#     3.2.4.3. HOST
#     3.2.4.4. NAMESPACE
#     3.2.4.5. CLASSPATH
#     3.2.4.6. LOCALCLASSPATH
#     3.2.4.7. CLASSNAME
#     3.2.4.8. INSTANCEPATH
#     3.2.4.9. LOCALINSTANCEPATH
#     3.2.4.10. INSTANCENAME
#     3.2.4.11. OBJECTPATH
#     3.2.4.12. KEYBINDING
#     3.2.4.13. KEYVALUE


class NamespacePath(CIMXMLTest):
    """
    <!ELEMENT NAMESPACEPATH (HOST,LOCALNAMESPACEPATH)>
    """

    def setUp(self):
        super(NamespacePath, self).setUp()

        self.xml.append(NAMESPACEPATH())


class LocalNamespacePath(CIMXMLTest):
    """
    <!ELEMENT LOCALNAMESPACEPATH (NAMESPACE+)>
    """

    def setUp(self):
        super(LocalNamespacePath, self).setUp()

        self.xml.append(LOCALNAMESPACEPATH())


class Host(CIMXMLTest):
    """
    <!ELEMENT HOST (#PCDATA)>
    """

    def setUp(self):
        super(Host, self).setUp()

        self.xml.append(cim_xml.HOST('leonardo'))


class Namespace(CIMXMLTest):
    """
    <!ELEMENT NAMESPACE EMPTY>
    <!ATTLIST NAMESPACE
        %CIMName;>
    """

    def setUp(self):
        super(Namespace, self).setUp()

        self.xml.append(cim_xml.NAMESPACE('root'))


class ClassPath(CIMXMLTest):
    """
    <!ELEMENT CLASSPATH (NAMESPACEPATH,CLASSNAME)>
    """

    def setUp(self):
        super(ClassPath, self).setUp()

        self.xml.append(cim_xml.CLASSPATH(NAMESPACEPATH(), CLASSNAME()))


class LocalClassPath(CIMXMLTest):
    """
    <!ELEMENT LOCALCLASSPATH (LOCALNAMESPACEPATH, CLASSNAME)>
    """

    def setUp(self):
        super(LocalClassPath, self).setUp()

        self.xml.append(cim_xml.LOCALCLASSPATH(
            LOCALNAMESPACEPATH(), CLASSNAME()))


class ClassName(CIMXMLTest):
    """
    <!ELEMENT CLASSNAME EMPTY>
    <!ATTLIST CLASSNAME
        %CIMName;>
    """

    def setUp(self):
        super(ClassName, self).setUp()

        self.xml.append(CLASSNAME())


class InstancePath(CIMXMLTest):
    """
    <!ELEMENT INSTANCEPATH (NAMESPACEPATH,INSTANCENAME)>
    """

    def setUp(self):
        super(InstancePath, self).setUp()

        self.xml.append(cim_xml.INSTANCEPATH(
            NAMESPACEPATH(), INSTANCENAME()))


class LocalInstancePath(CIMXMLTest):
    """
    <!ELEMENT LOCALINSTANCEPATH (LOCALNAMESPACEPATH,INSTANCENAME)>
    """

    def setUp(self):
        super(LocalInstancePath, self).setUp()

        self.xml.append(cim_xml.LOCALINSTANCEPATH(
            LOCALNAMESPACEPATH(), INSTANCENAME()))


class InstanceName(CIMXMLTest):
    """
    <!ELEMENT INSTANCENAME (KEYBINDING*|KEYVALUE?|VALUE.REFERENCE?)>
    <!ATTLIST INSTANCENAME
        %ClassName;>
    """

    def setUp(self):
        super(InstanceName, self).setUp()

        # Empty

        self.xml.append(cim_xml.INSTANCENAME('CIM_Pet', None))

        # KEYBINDING

        self.xml.append(INSTANCENAME())

        # KEYVALUE

        self.xml.append(cim_xml.INSTANCENAME(
            'CIM_Pet', cim_xml.KEYVALUE('FALSE', 'boolean')))

        # VALUE.REFERENCE

        self.xml.append(cim_xml.INSTANCENAME(
            'CIM_Pet',
            cim_xml.VALUE_REFERENCE(INSTANCENAME())))


class ObjectPath(CIMXMLTest):
    """
    <!ELEMENT OBJECTPATH (INSTANCEPATH|CLASSPATH)>
    """

    def setUp(self):
        super(ObjectPath, self).setUp()

        self.xml.append(cim_xml.OBJECTPATH(
            cim_xml.INSTANCEPATH(
                NAMESPACEPATH(), INSTANCENAME())))

        self.xml.append(cim_xml.OBJECTPATH(
            cim_xml.CLASSPATH(NAMESPACEPATH(), CLASSNAME())))


class KeyBinding(CIMXMLTest):
    """
    <!ELEMENT KEYBINDING (KEYVALUE|VALUE.REFERENCE)>
    <!ATTLIST KEYBINDING
        %CIMName;>
    """

    def setUp(self):
        super(KeyBinding, self).setUp()

        self.xml.append(cim_xml.KEYBINDING(
            'pet', cim_xml.KEYVALUE('dog', 'string')))

        self.xml.append(cim_xml.KEYBINDING(
            'CIM_Foo',
            cim_xml.VALUE_REFERENCE(
                cim_xml.CLASSPATH(NAMESPACEPATH(), CLASSNAME()))))


class KeyValue(CIMXMLTest):
    """
    <!ELEMENT KEYVALUE (#PCDATA)>
    <!ATTLIST KEYVALUE
        VALUETYPE    (string|boolean|numeric)  'string'
        %CIMType;    #IMPLIED>
    """

    def setUp(self):
        super(KeyValue, self).setUp()

        self.xml.append(cim_xml.KEYVALUE('dog', 'string'))
        self.xml.append(cim_xml.KEYVALUE('2', 'numeric'))
        self.xml.append(cim_xml.KEYVALUE('FALSE', 'boolean'))
        self.xml.append(cim_xml.KEYVALUE('2', 'numeric', 'uint16'))
        self.xml.append(cim_xml.KEYVALUE(None))


#################################################################
#     3.2.5. Object Definition Elements
#################################################################

#     3.2.5.1. CLASS
#     3.2.5.2. INSTANCE
#     3.2.5.3. QUALIFIER
#     3.2.5.4. PROPERTY
#     3.2.5.5. PROPERTY.ARRAY
#     3.2.5.6. PROPERTY.REFERENCE
#     3.2.5.7. METHOD
#     3.2.5.8. PARAMETER
#     3.2.5.9. PARAMETER.REFERENCE
#     3.2.5.10. PARAMETER.ARRAY
#     3.2.5.11. PARAMETER.REFARRAY
#     3.2.5.12. TABLECELL.DECLARATION
#     3.2.5.13. TABLECELL.REFERENCE
#     3.2.5.14. TABLEROW.DECLARATION
#     3.2.5.15. TABLE
#     3.2.5.16. TABLEROW


class Class(CIMXMLTest):
    """
    <!ELEMENT CLASS (QUALIFIER*,(PROPERTY|PROPERTY.ARRAY|PROPERTY.REFERENCE)*,
                     METHOD*)>
    <!ATTLIST CLASS
        %CIMName;
        %SuperClass;>
    """

    def setUp(self):
        super(Class, self).setUp()

        # Empty

        self.xml.append(cim_xml.CLASS('CIM_Foo'))

        # PROPERTY

        self.xml.append(cim_xml.CLASS(
            'CIM_Foo',
            properties=[cim_xml.PROPERTY('Dog', 'string',
                                         cim_xml.VALUE('Spotty'))]))

        # QUALIFIER + PROPERTY

        self.xml.append(cim_xml.CLASS(
            'CIM_Foo',
            properties=[cim_xml.PROPERTY('Dog', 'string',
                                         cim_xml.VALUE('Spotty'))],
            qualifiers=[cim_xml.QUALIFIER('IMPISH', 'string',
                                          cim_xml.VALUE('true'))]))

        # PROPERTY.ARRAY

        self.xml.append(cim_xml.CLASS(
            'CIM_Foo',
            properties=[cim_xml.PROPERTY_ARRAY('Dogs', 'string', None)]))

        # PROPERTY.REFERENCE

        self.xml.append(cim_xml.CLASS(
            'CIM_Foo',
            properties=[cim_xml.PROPERTY_REFERENCE('Dogs', None)]))

        # METHOD

        self.xml.append(cim_xml.CLASS(
            'CIM_Foo',
            methods=[cim_xml.METHOD('FooMethod')]))


class Instance(CIMXMLTest):
    """
    <!ELEMENT INSTANCE (QUALIFIER*,(PROPERTY|PROPERTY.ARRAY|
                                    PROPERTY.REFERENCE)*)>
    <!ATTLIST INSTANCE
         %ClassName;
         xml:lang   NMTOKEN  #IMPLIED>
    """

    def setUp(self):
        super(Instance, self).setUp()

        # Empty

        self.xml.append(cim_xml.INSTANCE('CIM_Foo', []))

        # PROPERTY

        self.xml.append(cim_xml.INSTANCE(
            'CIM_Foo',
            [cim_xml.PROPERTY('Dog', 'string', cim_xml.VALUE('Spotty')),
             cim_xml.PROPERTY('Cat', 'string', cim_xml.VALUE('Bella'))]))

        # PROPERTY + QUALIFIER

        self.xml.append(cim_xml.INSTANCE(
            'CIM_Foo',
            properties=[cim_xml.PROPERTY('Dog', 'string',
                                         cim_xml.VALUE('Spotty')),
                        cim_xml.PROPERTY('Cat', 'string',
                                         cim_xml.VALUE('Bella'))],
            qualifiers=[cim_xml.QUALIFIER('IMPISH', 'string',
                                          cim_xml.VALUE('true'))]))

        # PROPERTY.ARRAY

        self.xml.append(cim_xml.INSTANCE(
            'CIM_Pets',
            [cim_xml.PROPERTY_ARRAY(
                'Dogs',
                'string',
                cim_xml.VALUE_ARRAY([cim_xml.VALUE('Spotty'),
                                     cim_xml.VALUE('Bronte')])),
             cim_xml.PROPERTY_ARRAY(
                 'Cats',
                 'string',
                 cim_xml.VALUE_ARRAY([cim_xml.VALUE('Bella'),
                                      cim_xml.VALUE('Faux Lily')]))]))

        # PROPERTY.REFERENCE

        self.xml.append(cim_xml.INSTANCE(
            'CIM_Pets',
            [cim_xml.PROPERTY_REFERENCE(
                'Dog',
                cim_xml.VALUE_REFERENCE(cim_xml.CLASSNAME('CIM_Dog'))),
             cim_xml.PROPERTY_REFERENCE(
                 'Cat',
                 cim_xml.VALUE_REFERENCE(cim_xml.CLASSNAME('CIM_Cat')))]))


class Qualifier(CIMXMLTest):
    """
    <!ELEMENT QUALIFIER (VALUE | VALUE.ARRAY)>
    <!ATTLIST QUALIFIER
        %CIMName;
        %CIMType;              #REQUIRED
        %Propagated;
        %QualifierFlavor;
        xml:lang   NMTOKEN  #IMPLIED>
    """

    def setUp(self):
        super(Qualifier, self).setUp()

        # Note: DTD 2.2 allows qualifier to be empty

        # VALUE

        self.xml.append(cim_xml.QUALIFIER(
            'IMPISH', 'string', cim_xml.VALUE('true')))

        # VALUE + attributes

        self.xml.append(cim_xml.QUALIFIER(
            'Key', 'string', cim_xml.VALUE('true'),
            overridable='true'))

        self.xml.append(cim_xml.QUALIFIER(
            'Description', 'string', cim_xml.VALUE('blahblah'),
            translatable='true'))

        self.xml.append(cim_xml.QUALIFIER(
            'Version', 'string', cim_xml.VALUE('foorble'),
            tosubclass='false', translatable='true'))

        # VALUE.ARRAY

        self.xml.append(cim_xml.QUALIFIER(
            'LUCKYNUMBERS', 'uint32',
            cim_xml.VALUE_ARRAY([cim_xml.VALUE('1'), cim_xml.VALUE('2')])))


class Property(CIMXMLTest):
    """
    <!ELEMENT PROPERTY (QUALIFIER*,VALUE?)>
    <!ATTLIST PROPERTY
        %CIMName;
        %CIMType;           #REQUIRED
        %ClassOrigin;
        %Propagated;
        xml:lang   NMTOKEN  #IMPLIED>
    """

    def setUp(self):
        super(Property, self).setUp()

        # Empty

        self.xml.append(cim_xml.PROPERTY('PropertyName', 'string', None))

        # PROPERTY

        self.xml.append(cim_xml.PROPERTY(
            'PropertyName',
            'string',
            cim_xml.VALUE('dog')))

        # PROPERTY + attributes

        self.xml.append(cim_xml.PROPERTY(
            'PropertyName',
            'string',
            cim_xml.VALUE('dog'),
            propagated='true', class_origin='CIM_Pets'))

        # PROPERTY + QUALIFIER

        self.xml.append(cim_xml.PROPERTY(
            'PropertyName',
            'string',
            cim_xml.VALUE('dog'),
            qualifiers=[cim_xml.QUALIFIER('IMPISH', 'string',
                                          cim_xml.VALUE('true'))]))


class PropertyArray(CIMXMLTest):
    """
    <!ELEMENT PROPERTY.ARRAY (QUALIFIER*,VALUE.ARRAY?)>
    <!ATTLIST PROPERTY.ARRAY
       %CIMName;
       %CIMType;           #REQUIRED
       %ArraySize;
       %ClassOrigin;
       %Propagated;
       xml:lang   NMTOKEN  #IMPLIED>

    """

    def setUp(self):
        super(PropertyArray, self).setUp()

        # Empty

        self.xml.append(cim_xml.PROPERTY_ARRAY('Dogs', 'string'))

        # VALUE.ARRAY

        self.xml.append(cim_xml.PROPERTY_ARRAY(
            'Dogs',
            'string',
            cim_xml.VALUE_ARRAY([cim_xml.VALUE('Spotty'),
                                 cim_xml.VALUE('Bronte')])))

        # VALUE.ARRAY + attributes

        self.xml.append(cim_xml.PROPERTY_ARRAY(
            'Dogs',
            'string',
            cim_xml.VALUE_ARRAY([cim_xml.VALUE('Spotty'),
                                 cim_xml.VALUE('Bronte')]),
            array_size='2', class_origin='CIM_Dog'))

        self.xml.append(cim_xml.PROPERTY_ARRAY('Dogs', 'string', None))

        # QUALIFIER + VALUE.ARRAY

        self.xml.append(cim_xml.PROPERTY_ARRAY(
            'Dogs',
            'string',
            cim_xml.VALUE_ARRAY([cim_xml.VALUE('Spotty'),
                                 cim_xml.VALUE('Bronte')]),
            qualifiers=[cim_xml.QUALIFIER('IMPISH', 'string',
                                          cim_xml.VALUE('true'))]))


class PropertyReference(CIMXMLTest):
    """
    <!ELEMENT PROPERTY.REFERENCE (QUALIFIER*,VALUE.REFERENCE?)>
    <!ATTLIST PROPERTY.REFERENCE
        %CIMName;
        %ReferenceClass;
        %ClassOrigin;
        %Propagated;>
    """

    def setUp(self):
        super(PropertyReference, self).setUp()

        # Empty

        self.xml.append(cim_xml.PROPERTY_REFERENCE('Dogs', None))

        # VALUE.REFERENCE

        self.xml.append(cim_xml.PROPERTY_REFERENCE(
            'Dogs',
            cim_xml.VALUE_REFERENCE(cim_xml.CLASSNAME('CIM_Dog'))))

        # VALUE.REFERENCE + attributes

        self.xml.append(cim_xml.PROPERTY_REFERENCE(
            'Dogs',
            cim_xml.VALUE_REFERENCE(cim_xml.CLASSNAME('CIM_Dog')),
            reference_class='CIM_Dog', class_origin='CIM_Dog',
            propagated='true'))

        # QUALIFIER + VALUE.REFERENCE

        self.xml.append(cim_xml.PROPERTY_REFERENCE(
            'Dogs',
            cim_xml.VALUE_REFERENCE(cim_xml.CLASSNAME('CIM_Dog')),
            qualifiers=[cim_xml.QUALIFIER('IMPISH', 'string',
                                          cim_xml.VALUE('true'))]))


class Method(CIMXMLTest):
    """
    <!ELEMENT METHOD (QUALIFIER*,(PARAMETER|PARAMETER.REFERENCE|
                                  PARAMETER.ARRAY|PARAMETER.REFARRAY)*)>
    <!ATTLIST METHOD
        %CIMName;
        %CIMType;          #IMPLIED
        %ClassOrigin;
        %Propagated;>
    """

    def setUp(self):
        super(Method, self).setUp()

        # Empty

        self.xml.append(cim_xml.METHOD('FooMethod'))

        # PARAMETER

        self.xml.append(cim_xml.METHOD(
            'FooMethod',
            [cim_xml.PARAMETER('arg', 'string')]))

        # PARAMETER.REFERENCE

        self.xml.append(cim_xml.METHOD(
            'FooMethod',
            [cim_xml.PARAMETER_REFERENCE('arg', 'CIM_Foo')]))

        # PARAMETER.ARRAY

        self.xml.append(cim_xml.METHOD(
            'FooMethod',
            [cim_xml.PARAMETER_ARRAY('arg', 'string')]))

        # PARAMETER.REFARRAY

        self.xml.append(cim_xml.METHOD(
            'FooMethod',
            [cim_xml.PARAMETER_REFARRAY('arg', 'CIM_Foo')]))

        # PARAMETER + attributes

        self.xml.append(cim_xml.METHOD(
            'FooMethod',
            [cim_xml.PARAMETER('arg', 'string')],
            return_type='uint32',
            class_origin='CIM_Foo',
            propagated='true'))

        # QUALIFIER + PARAMETER

        self.xml.append(cim_xml.METHOD(
            'FooMethod',
            [cim_xml.PARAMETER('arg', 'string')],
            qualifiers=[cim_xml.QUALIFIER('IMPISH', 'string',
                                          cim_xml.VALUE('true'))]))


class Parameter(CIMXMLTest):
    """
    <!ELEMENT PARAMETER (QUALIFIER*)>
    <!ATTLIST PARAMETER
        %CIMName;
        %CIMType;      #REQUIRED>
    """

    def setUp(self):
        super(Parameter, self).setUp()

        # Empty

        self.xml.append(cim_xml.PARAMETER('arg', 'string'))

        # QUALIFIER

        self.xml.append(cim_xml.PARAMETER(
            'arg',
            'string',
            qualifiers=[cim_xml.QUALIFIER('IMPISH', 'string',
                                          cim_xml.VALUE('true'))]))


class ParameterReference(CIMXMLTest):
    """
    <!ELEMENT PARAMETER.REFERENCE (QUALIFIER*)>
    <!ATTLIST PARAMETER.REFERENCE
        %CIMName;
        %ReferenceClass;>
    """

    def setUp(self):
        super(ParameterReference, self).setUp()

        # Empty

        self.xml.append(cim_xml.PARAMETER_REFERENCE('arg'))

        # QUALIFIER + attributes

        self.xml.append(cim_xml.PARAMETER_REFERENCE(
            'arg',
            reference_class='CIM_Foo',
            qualifiers=[cim_xml.QUALIFIER('IMPISH', 'string',
                                          cim_xml.VALUE('true'))]))


class ParameterArray(CIMXMLTest):
    """
    <!ELEMENT PARAMETER.ARRAY (QUALIFIER*)>
    <!ATTLIST PARAMETER.ARRAY
        %CIMName;
        %CIMType;           #REQUIRED
        %ArraySize;>
    """

    def setUp(self):
        super(ParameterArray, self).setUp()

        # Empty

        self.xml.append(cim_xml.PARAMETER_ARRAY('arg', 'string'))

        # QUALIFIERS + attributes

        self.xml.append(cim_xml.PARAMETER_ARRAY(
            'arg',
            'string',
            array_size='0',
            qualifiers=[cim_xml.QUALIFIER('IMPISH', 'string',
                                          cim_xml.VALUE('true'))]))


class ParameterReferenceArray(CIMXMLTest):
    """
    <!ELEMENT PARAMETER.REFARRAY (QUALIFIER*)>
    <!ATTLIST PARAMETER.REFARRAY
        %CIMName;
        %ReferenceClass;
        %ArraySize;>
    """

    def setUp(self):
        super(ParameterReferenceArray, self).setUp()

        # Empty

        self.xml.append(cim_xml.PARAMETER_REFARRAY('arg'))

        # QUALIFIERS + attributes

        self.xml.append(cim_xml.PARAMETER_REFARRAY(
            'arg',
            reference_class='CIM_Foo',
            array_size='0',
            qualifiers=[cim_xml.QUALIFIER('IMPISH', 'string',
                                          cim_xml.VALUE('true'))]))


# New in v2.2 of the DTD

# TABLECELL.DECLARATION
# TABLECELL.REFERENCE
# TABLEROW.DECLARATION
# TABLE
# TABLEROW


#################################################################
#     3.2.6. Message Elements
#################################################################

#     3.2.6.1. MESSAGE
#     3.2.6.2. MULTIREQ
#     3.2.6.3. SIMPLEREQ
#     3.2.6.4. METHODCALL
#     3.2.6.5. PARAMVALUE
#     3.2.6.6. IMETHODCALL
#     3.2.6.7. IPARAMVALUE
#     3.2.6.8. MULTIRSP
#     3.2.6.9. SIMPLERSP
#     3.2.6.10. METHODRESPONSE
#     3.2.6.11. IMETHODRESPONSE
#     3.2.6.12. ERROR
#     3.2.6.13. RETURNVALUE
#     3.2.6.14. IRETURNVALUE
#     3.2.6.15 MULTIEXPREQ
#     3.2.6.16 SIMPLEEXPREQ
#     3.2.6.17 EXPMETHODCALL
#     3.2.6.18 MULTIEXPRSP
#     3.2.6.19 SIMPLEEXPRSP
#     3.2.6.20 EXPMETHODRESPONSE
#     3.2.6.21 EXPPARAMVALUE
#     3.2.6.22 RESPONSEDESTINATION
#     3.2.6.23 SIMPLEREQACK


class Message(CIMXMLTest):
    """
    <!ELEMENT MESSAGE (SIMPLEREQ | MULTIREQ | SIMPLERSP | MULTIRSP |
                       SIMPLEEXPREQ | MULTIEXPREQ | SIMPLEEXPRSP |
                       MULTIEXPRSP)>
    <!ATTLIST MESSAGE
        ID CDATA #REQUIRED
        PROTOCOLVERSION CDATA #REQUIRED>
    """

    def setUp(self):
        super(Message, self).setUp()

        # SIMPLEREQ

        self.xml.append(cim_xml.MESSAGE(
            cim_xml.SIMPLEREQ(
                cim_xml.IMETHODCALL(
                    'FooMethod',
                    LOCALNAMESPACEPATH())),
            '1001', '1.0'))

        # MULTIREQ

        self.xml.append(cim_xml.MESSAGE(
            cim_xml.MULTIREQ(
                [cim_xml.SIMPLEREQ(cim_xml.IMETHODCALL('FooMethod',
                                                       LOCALNAMESPACEPATH())),
                 cim_xml.SIMPLEREQ(cim_xml.IMETHODCALL('FooMethod',
                                                       LOCALNAMESPACEPATH()))]),
            '1001', '1.0'))

        # SIMPLERSP

        self.xml.append(cim_xml.MESSAGE(
            cim_xml.SIMPLERSP(
                cim_xml.IMETHODRESPONSE('FooMethod')),
            '1001', '1.0'))

        # MULTIRSP

        self.xml.append(cim_xml.MESSAGE(
            cim_xml.MULTIRSP(
                [cim_xml.SIMPLERSP(cim_xml.IMETHODRESPONSE('FooMethod')),
                 cim_xml.SIMPLERSP(cim_xml.IMETHODRESPONSE('FooMethod'))]),
            '1001', '1.0'))

        # TODO:

        # SIMPLEEXPREQ
        # MULTIEXPREQ
        # SIMPLEEXPRSP
        # MULTIEXPRSP


class MultiReq(CIMXMLTest):
    """
    <!ELEMENT MULTIREQ (SIMPLEREQ, SIMPLEREQ+)>
    """

    def setUp(self):
        super(MultiReq, self).setUp()

        self.xml.append(cim_xml.MULTIREQ(
            [cim_xml.SIMPLEREQ(cim_xml.IMETHODCALL('FooMethod',
                                                   LOCALNAMESPACEPATH())),
             cim_xml.SIMPLEREQ(cim_xml.IMETHODCALL('FooMethod',
                                                   LOCALNAMESPACEPATH()))]))


class MultiExpReq(CIMXMLTest):
    """
    <!ELEMENT MULTIEXPREQ (SIMPLEEXPREQ, SIMPLEEXPREQ+)>
    """

    def setUp(self):
        super(MultiExpReq, self).setUp()

        self.xml.append(cim_xml.MULTIEXPREQ(
            [cim_xml.SIMPLEEXPREQ(cim_xml.EXPMETHODCALL('FooMethod')),
             cim_xml.SIMPLEEXPREQ(cim_xml.EXPMETHODCALL('FooMethod'))]))


class SimpleReq(CIMXMLTest):
    """
    <!ELEMENT SIMPLEREQ (IMETHODCALL | METHODCALL)>
    """

    def setUp(self):
        super(SimpleReq, self).setUp()

        # IMETHODCALL

        self.xml.append(cim_xml.SIMPLEREQ(
            cim_xml.IMETHODCALL('FooIMethod', LOCALNAMESPACEPATH())))

        # METHODCALL

        self.xml.append(cim_xml.SIMPLEREQ(
            cim_xml.METHODCALL(
                'FooMethod',
                cim_xml.LOCALCLASSPATH(LOCALNAMESPACEPATH(), CLASSNAME()))))


class SimpleExpReq(CIMXMLTest):
    """
    <!ELEMENT SIMPLEEXPREQ (EXPMETHODCALL)>
    """

    def setUp(self):
        super(SimpleExpReq, self).setUp()

        self.xml.append(cim_xml.SIMPLEEXPREQ(
            cim_xml.EXPMETHODCALL('FooMethod')))


class IMethodCall(CIMXMLTest):
    """
    <!ELEMENT IMETHODCALL (LOCALNAMESPACEPATH, IPARAMVALUE*,
                           RESPONSEDESTINATION?)>
    <!ATTLIST IMETHODCALL
        %CIMName;>
    """

    def setUp(self):
        super(IMethodCall, self).setUp()

        self.xml.append(
            cim_xml.IMETHODCALL('FooMethod', LOCALNAMESPACEPATH()))

        self.xml.append(cim_xml.IMETHODCALL(
            'FooMethod2', LOCALNAMESPACEPATH(),
            [cim_xml.IPARAMVALUE('Dog', cim_xml.VALUE('Spottyfoot'))]))

        # TODO: RESPONSEDESTINATION


class MethodCall(CIMXMLTest):
    """
    <!ELEMENT METHODCALL ((LOCALINSTANCEPATH | LOCALCLASSPATH), PARAMVALUE*,
                          RESPONSEDESTINATION?>
    <!ATTLIST METHODCALL
        %CIMName;>
    """

    def setUp(self):
        super(MethodCall, self).setUp()

        # LOCALINSTANCEPATH

        self.xml.append(cim_xml.METHODCALL(
            'FooMethod',
            cim_xml.LOCALINSTANCEPATH(LOCALNAMESPACEPATH(), INSTANCENAME())))

        # LOCALCLASSPATH

        self.xml.append(cim_xml.METHODCALL(
            'FooMethod',
            cim_xml.LOCALCLASSPATH(LOCALNAMESPACEPATH(), CLASSNAME())))

        # PARAMVALUEs

        self.xml.append(cim_xml.METHODCALL(
            'FooMethod',
            cim_xml.LOCALINSTANCEPATH(LOCALNAMESPACEPATH(), INSTANCENAME()),
            [cim_xml.PARAMVALUE('Dog', cim_xml.VALUE('Spottyfoot'))]))

        # TODO: RESPONSEDESTINATION


class ExpMethodCall(CIMXMLTest):
    """
    <!ELEMENT EXPMETHODCALL (EXPPARAMVALUE*)>
    <!ATTLIST EXPMETHODCALL
        %CIMName;>
    """

    def setUp(self):
        super(ExpMethodCall, self).setUp()

        self.xml.append(cim_xml.EXPMETHODCALL('FooMethod'))

        self.xml.append(cim_xml.EXPMETHODCALL(
            'FooMethod',
            [cim_xml.EXPPARAMVALUE('Dog')]))


class ParamValue(CIMXMLTest):
    """
    <!ELEMENT PARAMVALUE (VALUE | VALUE.REFERENCE | VALUE.ARRAY |
                          VALUE.REFARRAY)?>
    <!ATTLIST PARAMVALUE
        %CIMName;
        %ParamType;  #IMPLIED>
    """

    def setUp(self):
        super(ParamValue, self).setUp()

        # Empty

        self.xml.append(cim_xml.PARAMVALUE('Pet'))

        # VALUE

        self.xml.append(cim_xml.PARAMVALUE(
            'Pet',
            cim_xml.VALUE('Dog'),
            'string'))

        # VALUE.REFERENCE

        self.xml.append(cim_xml.PARAMVALUE(
            'Pet',
            cim_xml.VALUE_REFERENCE(cim_xml.CLASSPATH(NAMESPACEPATH(),
                                                      CLASSNAME()))))

        # VALUE.ARRAY

        self.xml.append(cim_xml.PARAMVALUE(
            'Pet',
            cim_xml.VALUE_ARRAY([])))

        # VALUE.REFARRAY

        self.xml.append(cim_xml.PARAMVALUE(
            'Pet',
            cim_xml.VALUE_REFARRAY([])))


class IParamValue(CIMXMLTest):
    """
    <!ELEMENT IPARAMVALUE (VALUE | VALUE.ARRAY | VALUE.REFERENCE |
                           INSTANCENAME | CLASSNAME | QUALIFIER.DECLARATION |
                           CLASS | INSTANCE | VALUE.NAMEDINSTANCE)?>
    <!ATTLIST IPARAMVALUE
        %CIMName;>
    """

    def setUp(self):
        super(IParamValue, self).setUp()

        # Empty

        self.xml.append(cim_xml.IPARAMVALUE('Bird'))

        # VALUE

        self.xml.append(cim_xml.IPARAMVALUE(
            'Pet',
            cim_xml.VALUE('Dog')))

        # VALUE.ARRAY

        self.xml.append(cim_xml.IPARAMVALUE(
            'Pet',
            cim_xml.VALUE_ARRAY([])))

        # VALUE.REFERENCE

        self.xml.append(cim_xml.IPARAMVALUE(
            'Pet',
            cim_xml.VALUE_REFERENCE(
                cim_xml.CLASSPATH(NAMESPACEPATH(), CLASSNAME()))))

        # INSTANCENAME

        self.xml.append(cim_xml.IPARAMVALUE(
            'Pet',
            INSTANCENAME()))

        # CLASSNAME

        self.xml.append(cim_xml.IPARAMVALUE(
            'Pet',
            CLASSNAME()))

        # TODO: QUALIFIER.DECLARATION

        # CLASS

        self.xml.append(cim_xml.IPARAMVALUE(
            'Pet',
            cim_xml.CLASS('CIM_Foo')))

        # INSTANCE

        self.xml.append(cim_xml.IPARAMVALUE(
            'Pet',
            cim_xml.INSTANCE('CIM_Pet', [])))

        # VALUE.NAMEDINSTANCE

        self.xml.append(cim_xml.IPARAMVALUE(
            'Pet',
            cim_xml.VALUE_NAMEDINSTANCE(
                INSTANCENAME(),
                cim_xml.INSTANCE('CIM_Pet', []))))


class ExpParamValue(CIMXMLTest):
    """
    <!ELEMENT EXPPARAMVALUE (INSTANCE? | VALUE? | METHODRESPONSE? |
                             IMETHODRESPONSE?)>
    <!ATTLIST EXPPARAMVALUE
        %CIMName;
        %ParamType;  #IMPLIED>
    """

    def setUp(self):
        super(ExpParamValue, self).setUp()

        self.xml.append(cim_xml.EXPPARAMVALUE('FooParam'))

        self.xml.append(cim_xml.EXPPARAMVALUE(
            'FooParam',
            cim_xml.INSTANCE('CIM_Pet', [])))


class MultiRsp(CIMXMLTest):
    """
    <!ELEMENT MULTIRSP (SIMPLERSP, SIMPLERSP+)>
    """

    def setUp(self):
        super(MultiRsp, self).setUp()

        self.xml.append(
            cim_xml.MULTIRSP(
                [cim_xml.SIMPLERSP(cim_xml.IMETHODRESPONSE('FooMethod')),
                 cim_xml.SIMPLERSP(cim_xml.IMETHODRESPONSE('FooMethod'))]))


class MultiExpRsp(CIMXMLTest):
    """
    <!ELEMENT MULTIEXPRSP (SIMPLEEXPRSP, SIMPLEEXPRSP+)>
    """

    def setUp(self):
        super(MultiExpRsp, self).setUp()

        self.xml.append(
            cim_xml.MULTIEXPRSP(
                [cim_xml.SIMPLEEXPRSP(cim_xml.EXPMETHODRESPONSE('FooMethod')),
                 cim_xml.SIMPLEEXPRSP(cim_xml.EXPMETHODRESPONSE('FooMethod'))]))


class SimpleRsp(CIMXMLTest):
    """
    <!ELEMENT SIMPLERSP (METHODRESPONSE | IMETHODRESPONSE | SIMPLEREQACK>
    """

    def setUp(self):
        super(SimpleRsp, self).setUp()

        # METHODRESPONSE

        self.xml.append(
            cim_xml.SIMPLERSP(cim_xml.METHODRESPONSE('FooMethod')))

        # IMETHODRESPONSE

        self.xml.append(
            cim_xml.SIMPLERSP(cim_xml.IMETHODRESPONSE('FooMethod')))

        # TODO: SIMPLEREQACK


class SimpleExpRsp(CIMXMLTest):
    """
    <!ELEMENT SIMPLEEXPRSP (EXPMETHODRESPONSE)>
    """

    def setUp(self):
        super(SimpleExpRsp, self).setUp()

        self.xml.append(
            cim_xml.SIMPLEEXPRSP(cim_xml.EXPMETHODRESPONSE('FooMethod')))


class MethodResponse(CIMXMLTest):
    """
    <!ELEMENT METHODRESPONSE (ERROR | (RETURNVALUE?, PARAMVALUE*))>
    <!ATTLIST METHODRESPONSE
        %CIMName;>
    """

    def setUp(self):
        super(MethodResponse, self).setUp()

        # ERROR

        self.xml.append(
            cim_xml.METHODRESPONSE(
                'FooMethod',
                cim_xml.ERROR('123')))

        # Empty

        self.xml.append(cim_xml.METHODRESPONSE('FooMethod'))

        # RETURNVALUE

        self.xml.append(
            cim_xml.METHODRESPONSE(
                'FooMethod',
                cim_xml.PARAMVALUE('Dog', cim_xml.VALUE('Spottyfoot'))))

        # PARAMVALUE

        self.xml.append(
            cim_xml.METHODRESPONSE(
                'FooMethod',
                cim_xml.PARAMVALUE('Dog', cim_xml.VALUE('Spottyfoot'))))

        # RETURNVALUE + PARAMVALUE

        self.xml.append(
            cim_xml.METHODRESPONSE(
                'FooMethod',
                (cim_xml.RETURNVALUE(cim_xml.VALUE('Dog')),
                 cim_xml.PARAMVALUE('Dog', cim_xml.VALUE('Spottyfoot')))))


class ExpMethodResponse(CIMXMLTest):
    """
    <!ELEMENT EXPMETHODRESPONSE (ERROR | IRETURNVALUE?)>
    <!ATTLIST EXPMETHODRESPONSE
        %CIMName;>
    """

    def setUp(self):
        super(ExpMethodResponse, self).setUp()

        # Empty

        self.xml.append(cim_xml.EXPMETHODRESPONSE('FooMethod'))

        # ERROR

        self.xml.append(cim_xml.EXPMETHODRESPONSE(
            'FooMethod',
            cim_xml.ERROR('123')))

        # IRETURNVALUE

        self.xml.append(cim_xml.EXPMETHODRESPONSE(
            'FooMethod',
            cim_xml.IRETURNVALUE(cim_xml.VALUE('Dog'))))


class IMethodResponse(CIMXMLTest):
    """
    <!ELEMENT IMETHODRESPONSE (ERROR | IRETURNVALUE?)>
    <!ATTLIST IMETHODRESPONSE
        %CIMName;>
    """

    def setUp(self):
        super(IMethodResponse, self).setUp()

        # Empty

        self.xml.append(cim_xml.IMETHODRESPONSE('FooMethod'))

        # ERROR

        self.xml.append(cim_xml.IMETHODRESPONSE(
            'FooMethod',
            cim_xml.ERROR('123')))

        # IRETURNVALUE

        self.xml.append(cim_xml.IMETHODRESPONSE(
            'FooMethod',
            cim_xml.IRETURNVALUE(cim_xml.VALUE('Dog'))))


class Error(CIMXMLTest):
    """
    <!ELEMENT ERROR (INSTANCE*)>
    <!ATTLIST ERROR
        CODE CDATA #REQUIRED
        DESCRIPTION CDATA #IMPLIED>
    """

    def setUp(self):
        super(Error, self).setUp()

        self.xml.append(cim_xml.ERROR('1'))
        self.xml.append(cim_xml.ERROR('1', 'Foo not found'))
        # TODO: INSTANCE*


class ReturnValue(CIMXMLTest):
    """
    <!ELEMENT RETURNVALUE (VALUE | VALUE.REFERENCE)>
    <!ATTLIST RETURNVALUE
        %ParamType;     #IMPLIED>
    """

    def setUp(self):
        super(ReturnValue, self).setUp()

        # VALUE

        self.xml.append(cim_xml.RETURNVALUE(cim_xml.VALUE('Dog')))

        # VALUE.REFERENCE

        self.xml.append(cim_xml.RETURNVALUE(cim_xml.VALUE_REFERENCE(
            cim_xml.CLASSPATH(NAMESPACEPATH(), CLASSNAME()))))

        # TODO: PARAMTYPE


class IReturnValue(CIMXMLTest):
    """
    <!ELEMENT IRETURNVALUE (CLASSNAME* | INSTANCENAME* | VALUE* |
                            VALUE.OBJECTWITHPATH* |
                            VALUE.OBJECTWITHLOCALPATH* | VALUE.OBJECT* |
                            OBJECTPATH* | QUALIFIER.DECLARATION* |
                            VALUE.ARRAY? | VALUE.REFERENCE? | CLASS* |
                            INSTANCE* | VALUE.NAMEDINSTANCE*)>
    """

    def setUp(self):
        super(IReturnValue, self).setUp()

        # Empty

        self.xml.append(cim_xml.IRETURNVALUE(None))

        # CLASSNAME

        self.xml.append(cim_xml.IRETURNVALUE(
            CLASSNAME()))

        # INSTANCENAME

        self.xml.append(cim_xml.IRETURNVALUE(
            INSTANCENAME()))

        # VALUE

        self.xml.append(cim_xml.IRETURNVALUE(
            cim_xml.VALUE('Dog')))

        # VALUE.OBJECTWITHPATH

        self.xml.append(cim_xml.IRETURNVALUE(
            cim_xml.VALUE_OBJECTWITHPATH(
                cim_xml.CLASSPATH(NAMESPACEPATH(), CLASSNAME()),
                cim_xml.CLASS('CIM_Foo'))))

        # VALUE.OBJECTWITHLOCALPATH

        self.xml.append(cim_xml.IRETURNVALUE(
            cim_xml.VALUE_OBJECTWITHLOCALPATH(
                cim_xml.LOCALCLASSPATH(LOCALNAMESPACEPATH(), CLASSNAME()),
                cim_xml.CLASS('CIM_Foo'))))

        # VALUE.OBJECT

        self.xml.append(cim_xml.IRETURNVALUE(
            cim_xml.VALUE_OBJECT(cim_xml.INSTANCE('CIM_Pet', []))))

        # OBJECTPATH

        self.xml.append(cim_xml.IRETURNVALUE(
            cim_xml.OBJECTPATH(
                cim_xml.INSTANCEPATH(NAMESPACEPATH(), INSTANCENAME()))))

        # TODO: QUALIFIER.DECLARATION

        # VALUE.ARRAY

        self.xml.append(cim_xml.IRETURNVALUE(
            cim_xml.VALUE_ARRAY([])))

        # VALUE.REFERENCE

        self.xml.append(cim_xml.IRETURNVALUE(
            cim_xml.VALUE_REFERENCE(
                cim_xml.CLASSPATH(NAMESPACEPATH(), CLASSNAME()))))

        # CLASS

        self.xml.append(cim_xml.IRETURNVALUE(
            cim_xml.CLASS('CIM_Foo')))

        # INSTANCE

        self.xml.append(cim_xml.IRETURNVALUE(
            cim_xml.INSTANCE('CIM_Pet', [])))

        # VALUE.NAMEDINSTANCE

        self.xml.append(cim_xml.IRETURNVALUE(
            cim_xml.VALUE_NAMEDINSTANCE(
                INSTANCENAME(),
                cim_xml.INSTANCE('CIM_Pet', []))))


# pylint: disable=too-few-public-methods
class ResponseDestination(UnimplementedTest):
    """
    The RESPONSEDESTINATION element contains an instance that
    describes the desired destination for the response.

    <!ELEMENT RESPONSEDESTINATON (INSTANCE)>
    """


# pylint: disable=too-few-public-methods
class SimpleReqAck(UnimplementedTest):
    """

    The SIMPLEREQACK defines the acknowledgement response to a Simple
    CIM Operation asynchronous request. The ERROR subelement is used
    to report a fundamental error which prevented the asynchronous
    request from being initiated.

    <!ELEMENT SIMPLEREQACK (ERROR?)>
    <!ATTLIST SIMPLEREQACK
        INSTANCEID CDATA     #REQUIRED>
    """


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# Test case-insensitive dictionary implementation.
#

from __future__ import absolute_import

import unittest
import warnings
import six

from pywbem.cim_obj import NocaseDict


class TestInit(unittest.TestCase):

    def test_all(self):

        # Empty

        dic = NocaseDict()
        self.assertTrue(len(dic) == 0)

        dic = NocaseDict(None)
        self.assertTrue(len(dic) == 0)

        dic = NocaseDict(list())
        self.assertTrue(len(dic) == 0)

        dic = NocaseDict(tuple())
        self.assertTrue(len(dic) == 0)

        dic = NocaseDict(dict())
        self.assertTrue(len(dic) == 0)

        dic = NocaseDict(dic)
        self.assertTrue(len(dic) == 0)

        # Initialise from iterable

        dic = NocaseDict([('Dog', 'Cat'), ('Budgie', 'Fish')])
        self.assertTrue(len(dic) == 2)
        self.assertTrue(dic['Dog'] == 'Cat' and dic['Budgie'] == 'Fish')

        dic = NocaseDict((('Dog', 'Cat'), ('Budgie', 'Fish')))
        self.assertTrue(len(dic) == 2)
        self.assertTrue(dic['Dog'] == 'Cat' and dic['Budgie'] == 'Fish')

        # Initialise from dictionary

        dic = NocaseDict({'Dog': 'Cat', 'Budgie': 'Fish'})
        self.assertTrue(len(dic) == 2)
        self.assertTrue(dic['Dog'] == 'Cat' and dic['Budgie'] == 'Fish')

        # Initialise from kwargs

        dic = NocaseDict(Dog='Cat', Budgie='Fish')
        self.assertTrue(len(dic) == 2)
        self.assertTrue(dic['Dog'] == 'Cat' and dic['Budgie'] == 'Fish')

        # Initialise from iterable and kwargs

        dic = NocaseDict([('Dog', 'Cat'), ], Budgie='Fish')
        self.assertTrue(len(dic) == 2)
        self.assertTrue(dic['Dog'] == 'Cat' and dic['Budgie'] == 'Fish')

        dic = NocaseDict((('Dog', 'Cat'),), Budgie='Fish')
        self.assertTrue(len(dic) == 2)
        self.assertTrue(dic['Dog'] == 'Cat' and dic['Budgie'] == 'Fish')

        # Initialise from dictionary and kwargs

        dic = NocaseDict({'Dog': 'Cat'}, Budgie='Fish')
        self.assertTrue(len(dic) == 2)
        self.assertTrue(dic['Dog'] == 'Cat' and dic['Budgie'] == 'Fish')

        # Initialise from unsupported object type

        try:
            dic = NocaseDict('illegal')
        except TypeError:
            pass
        else:
            self.fail("TypeError was unexpectedly not thrown.")

        # Initialise with too many positional arguments

        try:
            dic = NocaseDict(list(), list())
        except TypeError:
            pass
        else:
            self.fail("TypeError was unexpectedly not thrown.")


class BaseTest(unittest.TestCase):

    def setUp(self):
        self.dic = NocaseDict()
        self.dic['Dog'] = 'Cat'
        self.dic['Budgie'] = 'Fish'


class TestGetitem(BaseTest):

    def test_all(self):
        self.assertTrue(self.dic['dog'] == 'Cat')
        self.assertTrue(self.dic['DOG'] == 'Cat')

        try:
            self.dic['notfound']
        except KeyError:
            pass
        else:
            self.fail("KeyError was unexpectedly not thrown.")


class TestLen(BaseTest):

    def test_all(self):
        self.assertTrue(len(self.dic) == 2)


class TestSetitem(BaseTest):

    def test_all(self):

        self.dic['DOG'] = 'Kitten'
        self.assertTrue(self.dic['DOG'] == 'Kitten')
        self.assertTrue(self.dic['Dog'] == 'Kitten')
        self.assertTrue(self.dic['dog'] == 'Kitten')

        # Check that using a non-string key raises an exception

        try:
            self.dic[1234] = '1234'
        except TypeError:
            pass
        else:
            self.fail('TypeError expected')


class TestDelitem(BaseTest):

    def test_all(self):
        del self.dic['DOG']
        del self.dic['budgie']
        self.assertTrue(self.dic.keys() == [])

        try:
            del self.dic['notfound']
        except KeyError:
            pass
        else:
            self.fail("KeyError was unexpectedly not thrown.")


class TestHasKey(BaseTest):

    def test_all(self):
        self.assertTrue('DOG' in self.dic)
        self.assertTrue('budgie' in self.dic)
        self.assertTrue(1234 not in self.dic)


class TestKeys(BaseTest):

    def test_all(self):
        keys = self.dic.keys()
        animals = ['Budgie', 'Dog']
        for ani in animals:
            self.assertTrue(ani in keys)
            keys.remove(ani)
        self.assertTrue(keys == [])


class TestValues(BaseTest):

    def test_all(self):
        values = self.dic.values()
        animals = ['Cat', 'Fish']
        for ani in animals:
            self.assertTrue(ani in values)
            values.remove(ani)
        self.assertTrue(values == [])


class TestItems(BaseTest):

    def test_all(self):
        items = self.dic.items()
        animals = [('Dog', 'Cat'), ('Budgie', 'Fish')]
        for ani in animals:
            self.assertTrue(ani in items)
            items.remove(ani)
        self.assertTrue(items == [])


class TestClear(BaseTest):

    def test_all(self):
        self.dic.clear()
        self.assertTrue(len(self.dic) == 0)


class TestUpdate(BaseTest):

    def test_all(self):
        self.dic.clear()
        self.dic.update({'Chicken': 'Ham'})
        self.assertTrue(self.dic.keys() == ['Chicken'])
        self.assertTrue(self.dic.values() == ['Ham'])
        self.dic.clear()
        self.dic.update({'Chicken': 'Ham'}, {'Dog': 'Cat'})
        keys = self.dic.keys()
        vals = self.dic.values()
        keys = list(keys)
        vals = list(vals)
        keys.sort()
        vals.sort()
        self.assertTrue(keys == ['Chicken', 'Dog'])
        self.assertTrue(vals == ['Cat', 'Ham'])
        self.dic.update([('Chicken', 'Egg')], {'Fish': 'Eel'})
        self.assertTrue(self.dic['chicken'] == 'Egg')
        self.assertTrue(self.dic['fish'] == 'Eel')
        self.dic.update({'Fish': 'Salmon'}, Cow='Beef')
        self.assertTrue(self.dic['fish'] == 'Salmon')
        self.assertTrue(self.dic['Cow'] == 'Beef')
        self.assertTrue(self.dic['COW'] == 'Beef')
        self.assertTrue(self.dic['cow'] == 'Beef')


class TestCopy(BaseTest):

    def test_all(self):
        cp = self.dic.copy()
        self.assertEqual(cp, self.dic)
        self.assertTrue(isinstance(cp, NocaseDict))
        cp['Dog'] = 'Kitten'
        self.assertTrue(self.dic['Dog'] == 'Cat')
        self.assertTrue(cp['Dog'] == 'Kitten')


class TestGet(BaseTest):

    def test_all(self):
        self.assertTrue(self.dic.get('Dog', 'Chicken') == 'Cat')
        self.assertTrue(self.dic.get('Ningaui') is None)
        self.assertTrue(self.dic.get('Ningaui', 'Chicken') == 'Chicken')


class TestSetDefault(BaseTest):

    def test_all(self):
        self.dic.setdefault('Dog', 'Kitten')
        self.assertTrue(self.dic['Dog'] == 'Cat')
        self.dic.setdefault('Ningaui', 'Chicken')
        self.assertTrue(self.dic['Ningaui'] == 'Chicken')


class TestPopItem(BaseTest):

    def test_all(self):
        pass


# TODO: swapcase2() is also defined in test_cim_obj.py. Consolidate.
def swapcase2(text):
    """Returns text, where every other character has been changed to swap
    its lexical case. For strings that contain at least one letter, the
    returned string is guaranteed to be different from the input string."""
    text_cs = ''
    i = 0
    for c in text:
        if i % 2 != 0:
            c = c.swapcase()
        text_cs += c
        i += 1
    return text_cs


class TestEqual(BaseTest):

    def assertDictEqual(self, dic1, dic2, msg):

        self.assertTrue(dic1 == dic2, msg)
        self.assertFalse(dic1 != dic2, msg)

        self.assertTrue(dic2 == dic1, msg)
        self.assertFalse(dic2 != dic1, msg)

    def assertDictNotEqual(self, dic1, dic2, msg):

        self.assertTrue(dic1 != dic2, msg)
        self.assertFalse(dic1 == dic2, msg)

        self.assertTrue(dic2 != dic1, msg)
        self.assertFalse(dic2 == dic1, msg)

    def run_test_dicts(self, base_dict, test_dicts):

        for test_dict, relation, comment in test_dicts:
            if relation == 'eq':
                self.assertDictEqual(test_dict, base_dict,
                                     "Expected test_dict == base_dict:\n"
                                     "  test case: %s\n"
                                     "  test_dict: %r\n"
                                     "  base_dict: %r" %
                                     (comment, test_dict, base_dict))
            elif relation == 'ne':
                self.assertDictNotEqual(test_dict, base_dict,
                                        "Expected test_dict != base_dict:\n"
                                        "  test case: %s\n"
                                        "  test_dict: %r\n"
                                        "  base_dict: %r" %
                                        (comment, test_dict, base_dict))
            else:
                raise AssertionError("Internal Error: Invalid relation %s"
                                     "specified in testcase: %s" %
                                     (relation, comment))

    def test_all(self):

        # The base dictionary that is used for all comparisons
        base_dict = dict({'Budgie': 'Fish', 'Dog': 'Cat'})

        # Test dictionaries to test against the base dict, as a list of
        # tuple(dict, relation, comment), with relation being the expected
        # comparison relation, and one of ('eq', 'ne').
        test_dicts = [

            (dict({'Budgie': 'Fish', 'Dog': 'Cat'}),
             'eq',
             'Same'),

            (dict({'Budgie': 'Fish'}),
             'ne',
             'Higher key missing, shorter size'),

            (dict({'Dog': 'Cat'}),
             'ne',
             'Lower key missing, shorter size'),

            (dict({'Budgie': 'Fish', 'Curly': 'Snake', 'Cozy': 'Dog'}),
             'ne',
             'First non-matching key is less. But longer size!'),

            (dict({'Alf': 'F', 'Anton': 'S', 'Aussie': 'D'}),
             'ne',
             'Only non-matching keys that are less. But longer size!'),

            (dict({'Budgio': 'Fish'}),
             'ne',
             'First non-matching key is greater. But shorter size!'),

            (dict({'Zoe': 'F'}),
             'ne',
             'Only non-matching keys that are greater. But shorter size!'),

            (dict({'Budgie': 'Fish', 'Curly': 'Snake'}),
             'ne',
             'Same size. First non-matching key is less'),

            (dict({'Alf': 'F', 'Anton': 'S'}),
             'ne',
             'Same size. Only non-matching keys that are less'),

            (dict({'Zoe': 'F', 'Zulu': 'S'}),
             'ne',
             'Same size. Only non-matching keys that are greater'),

            (dict({'Budgie': 'Fish', 'Dog': 'Car'}),
             'ne',
             'Same size, only matching keys. First non-matching value is less'),

            (dict({'Budgie': 'Fish', 'Dog': 'Caz'}),
             'ne',
             'Same size, only matching keys. First non-matching value is grt.'),
        ]

        # First, run these tests against a standard dictionary to verify
        # that the test case definitions conform to that
        self.run_test_dicts(base_dict, test_dicts)

        # Then, transform these tests to NocaseDict and run them again
        TEST_CASE_INSENSITIVITY = True
        base_ncdict = NocaseDict(base_dict)
        test_ncdicts = []
        for test_dict, relation, comment in test_dicts:
            test_ncdict = NocaseDict()
            for key in test_dict:
                if TEST_CASE_INSENSITIVITY:
                    nc_key = swapcase2(key)
                else:
                    nc_key = key
                test_ncdict[nc_key] = test_dict[key]
            test_ncdicts.append((test_ncdict, relation, comment))
        self.run_test_dicts(base_ncdict, test_ncdicts)


class TestOrdering(BaseTest):
    """Verify that ordering comparisons between NocaseDict instances
    issue a deprecation warning, and for Python 3, in addition raise
    TypeError."""

    def assertWarning(self, comp_str):
        with warnings.catch_warnings(record=True) as wlist:
            warnings.simplefilter("always")
            if six.PY2:
                eval(comp_str)
            else:
                try:
                    eval(comp_str)
                except TypeError as exc:
                    msg = str(exc)
                    if "not supported between instances" not in msg and \
                            "unorderable types" not in msg:
                        self.fail("Applying ordering to a dictionary in "
                                  "Python 3 did raise TypeError but with an "
                                  "unexpected message: %s" % msg)
                except Exception as exc:
                    msg = str(exc)
                    self.fail("Applying ordering to a dictionary in Python 3 "
                              "did not raise TypeError, but %s: %s" %
                              (exc.__class__.__name__, msg))
                else:
                    self.fail("Applying ordering to a dictionary in Python 3 "
                              "succeeded (should not happen)")
            assert len(wlist) >= 1
            assert issubclass(wlist[-1].category, DeprecationWarning)
            assert "deprecated" in str(wlist[-1].message)

    def test_all(self):
        self.assertWarning("self.dic < self.dic")
        self.assertWarning("self.dic <= self.dic")
        self.assertWarning("self.dic > self.dic")
        self.assertWarning("self.dic >= self.dic")


class TestContains(BaseTest):

    def test_all(self):
        self.assertTrue('dog' in self.dic)
        self.assertTrue('Dog' in self.dic)
        self.assertTrue('Cat' not in self.dic)


class TestForLoop(BaseTest):

    def test_all(self):
        keys = set()
        for key in self.dic:
            keys.add(key)
        self.assertTrue(keys, set(['Budgie', 'Dog']))


class TestIterkeys(BaseTest):

    def test_all(self):
        keys = set()
        for key in self.dic.iterkeys():
            keys.add(key)
        self.assertTrue(keys, set(['Budgie', 'Dog']))


class TestItervalues(BaseTest):

    def test_all(self):
        vals = set()
        for val in self.dic.itervalues():
            vals.add(val)
        self.assertTrue(vals, set(['Cat', 'Fish']))


class TestIteritems(BaseTest):

    def test_all(self):
        items = set()
        for item in self.dic.iteritems():
            items.add(item)
        self.assertTrue(items, set([('Budgie', 'Fish'), ('Dog', 'Cat')]))


class TestRepr(unittest.TestCase):

    def test_reliable_order(self):
        """Test that repr() has a reliable result despite different orders of
        insertion into the dictionary."""

        dic1 = NocaseDict()
        dic1['Budgie'] = 'Fish'
        dic1['Dog'] = 'Cat'
        dic1['Foo'] = 'Bla'

        dic2 = NocaseDict()
        dic2['Foo'] = 'Bla'
        dic2['Dog'] = 'Cat'
        dic2['Budgie'] = 'Fish'

        self.assertEqual(repr(dic1), repr(dic2))


if __name__ == '__main__':
    unittest.main()
